@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2007
+ * Copyright (C) 2002-2010
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,11 +20,11 @@
 
 #include "utils.h"
 #include "check.h"
-#include "application.h"
 #include "project.h"
 
 #include "connection.h"
 
+
 /**
  * UCTcpState:
  * @UC_TCP_STATE_CONNECTING: We are not yet connected.
@@ -129,6 +129,7 @@ struct _UCConn
 		uc_check_ignore_item_get_value ()\
 	)
 
+
 #ifdef ENABLE_GNUTLS
 static gnutls_certificate_client_credentials xcred_gnutls;
 #endif
@@ -139,10 +140,12 @@ static UCConn *uc_conn_new (const gchar * proto, gchar * host,
 			    guint port, gchar * buffer,
 			    gboolean header_request);
 static void uc_conn_disconnect (UCConn * conn);
-static void uc_conn_free (UCConn * conn);
-static UCTcpStatus connect_nonblock (const gint soc, struct sockaddr_in addr);
+static void uc_conn_free (UCConn **conn);
+static UCTcpStatus connect_nonblock (const gint soc, struct sockaddr *addr,
+                                     socklen_t addrlen);
 static gboolean uc_conn_timeout_cb (gpointer data);
 
+
 /**
  * uc_conn_init:
  * 
@@ -159,6 +162,7 @@ uc_conn_init (void)
 #endif
 }
 
+
 /**
  * uc_conn_free:
  * @conn: #UCConn object to free
@@ -166,18 +170,19 @@ uc_conn_init (void)
  * Free a #UCConn structure.
  */
 static void
-uc_conn_free (UCConn * conn)
+uc_conn_free (UCConn **conn)
 {
-  uc_conn_disconnect (conn);
+  uc_conn_disconnect (*conn);
 
-  if (conn->array != NULL)
-    g_array_free (conn->array, TRUE), conn->array = NULL;
-  if (conn->hostname != NULL)
-    g_free (conn->hostname), conn->hostname = NULL;
+  if ((*conn)->array != NULL)
+    g_array_free ((*conn)->array, TRUE), (*conn)->array = NULL;
+  if ((*conn)->hostname != NULL)
+    g_free ((*conn)->hostname), (*conn)->hostname = NULL;
 
-  g_free (conn), conn = NULL;
+  g_free (*conn), *conn = NULL;
 }
 
+
 /**
  * uc_conn_new:
  * @proto: The protocol.
@@ -196,6 +201,7 @@ uc_conn_new (const gchar * proto, gchar * host, guint port, gchar * buffer,
 {
   UCConn *conn = NULL;
 
+
   conn = g_new0 (UCConn, 1);
   conn->thread_end = FALSE;
   conn->thread_ftp_end = FALSE;
@@ -212,40 +218,42 @@ uc_conn_new (const gchar * proto, gchar * host, guint port, gchar * buffer,
   conn->array = g_array_new (FALSE, TRUE, 1);
   conn->request = buffer;
 
-  if (!strcmp (proto, UC_PROTOCOL_FTP))
-    {
-      conn->use_ftp = TRUE;
-      conn->port = atoi (UC_URL_DEFAULT_FTP_PORT);
-    }
+  if (strcmp (proto, UC_PROTOCOL_FTP) == 0)
+  {
+    conn->use_ftp = TRUE;
+    conn->port = atoi (UC_URL_DEFAULT_FTP_PORT);
+  }
 #ifdef ENABLE_GNUTLS
-  if (!strcmp (proto, UC_PROTOCOL_HTTPS))
+  if (strcmp (proto, UC_PROTOCOL_HTTPS) == 0)
+  {
+    const gint cert_type_priority[3] =
     {
-      const gint cert_type_priority[3] = {
-	GNUTLS_CRT_X509,
-	GNUTLS_CRT_OPENPGP,
-	0
-      };
+      GNUTLS_CRT_X509,
+      GNUTLS_CRT_OPENPGP,
+      0
+    };
 
-      conn->use_gnutls = TRUE;
-      conn->port = atoi (UC_URL_DEFAULT_SSL_PORT);
+    conn->use_gnutls = TRUE;
+    conn->port = atoi (UC_URL_DEFAULT_SSL_PORT);
 
-      gnutls_init (&conn->session_gnutls, GNUTLS_CLIENT);
-      gnutls_set_default_priority (conn->session_gnutls);
-      gnutls_certificate_type_set_priority (conn->session_gnutls,
-					    cert_type_priority);
-      gnutls_credentials_set (conn->session_gnutls, GNUTLS_CRD_CERTIFICATE,
-			      xcred_gnutls);
-    }
+    gnutls_init (&conn->session_gnutls, GNUTLS_CLIENT);
+    gnutls_set_default_priority (conn->session_gnutls);
+    gnutls_certificate_type_set_priority (conn->session_gnutls,
+                                          cert_type_priority);
+    gnutls_credentials_set (conn->session_gnutls, GNUTLS_CRD_CERTIFICATE,
+                            xcred_gnutls);
+  }
   else
-    {
-      conn->session_gnutls = NULL;
-      conn->use_gnutls = FALSE;
-    }
+  {
+    conn->session_gnutls = NULL;
+    conn->use_gnutls = FALSE;
+  }
 #endif
 
   return conn;
 }
 
+
 /**
  * uc_conn_timeout_cb:
  * @data: a #UCConn oject.
@@ -255,14 +263,12 @@ uc_conn_new (const gchar * proto, gchar * host, guint port, gchar * buffer,
 static gboolean
 uc_conn_timeout_cb (gpointer data)
 {
-  UCConn *conn = (UCConn *) data;
-
-  pthread_cancel (conn->thread_id);
-  conn->thread_end = TRUE;
+  ((UCConn *)data)->thread_end = TRUE;
 
   return FALSE;
 }
 
+
 /**
  * uc_conn_disconnect:
  * @conn: A #UCConn object.
@@ -272,31 +278,31 @@ uc_conn_timeout_cb (gpointer data)
 static void
 uc_conn_disconnect (UCConn * conn)
 {
-
   pthread_cancel (conn->thread_id);
 
 #ifdef ENABLE_GNUTLS
   if (conn->use_gnutls)
-    {
-      gnutls_bye (conn->session_gnutls, GNUTLS_SHUT_RDWR);
-      gnutls_deinit (conn->session_gnutls);
-      conn->session_gnutls = NULL;
-    }
+  {
+    gnutls_bye (conn->session_gnutls, GNUTLS_SHUT_RDWR);
+    gnutls_deinit (conn->session_gnutls);
+    conn->session_gnutls = NULL;
+  }
 #endif
 
   if (conn->socket > 0)
-    {
-      shutdown (conn->socket, SHUT_RDWR);
-      close (conn->socket), conn->socket = 0;
-    }
+  {
+    shutdown (conn->socket, SHUT_RDWR);
+    close (conn->socket), conn->socket = 0;
+  }
 
   if (conn->ftp_passive_socket)
-    {
-      shutdown (conn->ftp_passive_socket, SHUT_RDWR);
-      close (conn->ftp_passive_socket), conn->ftp_passive_socket = 0;
-    }
+  {
+    shutdown (conn->ftp_passive_socket, SHUT_RDWR);
+    close (conn->ftp_passive_socket), conn->ftp_passive_socket = 0;
+  }
 }
 
+
 /**
  * uc_server_get_response:
  * @proto: The protocol.
@@ -320,6 +326,7 @@ uc_server_get_response (const gchar * proto, gchar * host, const guint port,
   GArray *array = NULL;
   guint timeout_id = 0;
 
+
   uc_check_ignore_item_set_value (FALSE);
   if (uc_check_cancel_get_value ())
     return NULL;
@@ -327,59 +334,53 @@ uc_server_get_response (const gchar * proto, gchar * host, const guint port,
   if (strlen (uc_project_get_proxy_host ()) > 0
       && (g_ascii_strcasecmp (host, "localhost") != 0
 	  && strstr (host, "127.0.0.") == NULL))
-
-    {
-      host_real = uc_project_get_proxy_host ();
-      port_real = uc_project_get_proxy_port ();
-    }
+  {
+    host_real = uc_project_get_proxy_host ();
+    port_real = uc_project_get_proxy_port ();
+  }
   else
-    {
-      host_real = host;
-      port_real = port;
-    }
+  {
+    host_real = host;
+    port_real = port;
+  }
 
+  // Connect
   conn = uc_conn_new (proto, host_real, port_real, buffer, header_request);
-
-  /* connect */
   conn->thread_end = FALSE;
   pthread_create (&conn->thread_id, NULL, connect_thread, conn);
   pthread_detach (conn->thread_id);
 
   UC_THREAD_WAIT (conn->thread_end, FALSE);
 
-  /* check was cancelled */
-  if (UC_CHECK_CANCELED)
-    pthread_cancel (conn->thread_id);
-  /* send client request and read server response */
-  else if (conn->tcp_status == UC_TCP_STATUS_OK)
+  // Connection OK : send client request and read server response
+  if (!UC_CHECK_CANCELED && conn->tcp_status == UC_TCP_STATUS_OK)
+  {
+    conn->thread_end = FALSE;
+    pthread_create (&conn->thread_id, NULL, read_thread, conn);
+    pthread_detach (conn->thread_id);
+
+    timeout_id = g_timeout_add (uc_project_get_check_timeout () * 1000,
+                                uc_conn_timeout_cb, conn);
+
+    UC_THREAD_WAIT (conn->thread_end, FALSE);
+    if (timeout_id != 0)
+      g_source_remove (timeout_id), timeout_id = 0;
+
+    // Read OK : get fill array with the server response
+    if (!UC_CHECK_CANCELED && conn->tcp_status == UC_TCP_STATUS_OK)
     {
-      conn->thread_end = FALSE;
-      pthread_create (&conn->thread_id, NULL, read_thread, conn);
-      pthread_detach (conn->thread_id);
-
-      timeout_id = g_timeout_add (uc_project_get_check_timeout () * 1000,
-				  uc_conn_timeout_cb, conn);
-
-      UC_THREAD_WAIT (conn->thread_end, FALSE);
-      if (timeout_id != 0)
-	g_source_remove (timeout_id), timeout_id = 0;
-
-      /* check was cancelled */
-      if (UC_CHECK_CANCELED)
-	pthread_cancel (conn->thread_id);
-      /* ok, get fill array with the server response */
-      else if (conn->tcp_status == UC_TCP_STATUS_OK)
-	{
-	  array = g_array_new (FALSE, TRUE, 1);
-	  g_array_append_vals (array, conn->array->data, conn->array->len);
-	}
+      array = g_array_new (FALSE, TRUE, 1);
+      g_array_append_vals (array, conn->array->data, conn->array->len);
     }
+  }
 
-  uc_conn_free (conn);
+  uc_conn_free (&conn);
 
   return array;
 }
 
+
+// FIXME rewrite all this function
 /**
  * read_thread:
  * @data: Pointer on a #UCConn structure.
@@ -396,209 +397,213 @@ read_thread (void *data)
   gchar *tmp = NULL;
   gchar buffer[UC_BUFFER_LEN + 1];
 
-#ifdef ENABLE_GNUTLS
-  /* HTTPS */
-  if (conn->use_gnutls)
-    {
-      gnutls_record_send (conn->session_gnutls, conn->request,
-			  strlen (conn->request));
-      while ((nb_read = gnutls_record_recv (conn->session_gnutls, buffer,
-					    UC_BUFFER_LEN)) > 0)
-	g_array_append_vals (conn->array, buffer, nb_read);
 
-      conn->tcp_status = (nb_read == 0) ?
-	UC_TCP_STATUS_OK : UC_TCP_STATUS_ERROR;
-      goto handler_exit;
+#ifdef ENABLE_GNUTLS
+  // HTTPS
+  if (conn->use_gnutls)
+  {
+    gnutls_record_send (conn->session_gnutls, conn->request,
+                        strlen (conn->request));
+    while ((nb_read = gnutls_record_recv (conn->session_gnutls, buffer,
+                                          UC_BUFFER_LEN)) > 0)
+    {
+      if (conn->header_request && (tmp = strstr (buffer, "\r\n\r\n")))
+      {
+        *tmp = '\0';
+        g_array_append_vals (conn->array, buffer, strlen (buffer));
+        nb_read = 0;
+        break;
+      }
+
+      g_array_append_vals (conn->array, buffer, nb_read);
     }
 
-  /* FTP */
+    conn->tcp_status = (nb_read == 0) ? UC_TCP_STATUS_OK : UC_TCP_STATUS_ERROR;
+    goto handler_exit;
+  }
+  // FTP
   else if (conn->use_ftp)
 #else
   if (conn->use_ftp)
 #endif
+  {
+    guint temp[6];
+    gboolean have_data = FALSE;
+    gchar r_user[] = "USER anonymous\r\n";
+    gchar r_pass[] = "PASS gurlchecker@gurlchecker.labs.libre-entreprise.org\r\n";
+    gchar r_pasv[] = "PASV\r\n";
+    gchar r_quit[] = "QUIT\r\n";
+    gchar *port = NULL;
+    gchar *request = NULL;
+    pthread_t thread;
+
+
+    // Read banner
+    while ((nb_read = read (conn->socket, buffer, UC_BUFFER_LEN)) > 0)
     {
-      guint temp[6];
-      gboolean have_data = FALSE;
-      gchar r_user[] = "USER anonymous\n";
-      gchar r_pass[] =
-	"PASS gurlchecker@gurlchecker.labs.libre-entreprise.org\n";
-      gchar r_pasv[] = "PASV\n";
-      gchar r_quit[] = "QUIT\n";
-      gchar *port = NULL;
-      gchar *request = NULL;
-      pthread_t thread;
-
-      /* read banner */
-      while ((nb_read = read (conn->socket, buffer, UC_BUFFER_LEN)) > 0)
-	{
-	  buffer[nb_read] = '\0';
-	  if (strstr (buffer, "220 "))
-	    break;
-	}
-
-      /* send USER command */
-      write (conn->socket, r_user, strlen (r_user));
-      sleep (1);
-      nb_read = read (conn->socket, buffer, UC_BUFFER_LEN);
-
-      /* 230: User logged in, proceed */
-      if (uc_utils_ftp_code_search (buffer, "230", nb_read))
-	;
-      /* 332: Need account for login */
-      else if (uc_utils_ftp_code_search (buffer, "332", nb_read))
-	{
-	  g_warning ("FTP: Need account for login (%s)", conn->hostname);
-	  goto handler_error;
-	}
-      /* 331: User name okay, need password */
-      else if (uc_utils_ftp_code_search (buffer, "331", nb_read))
-	{
-	  /* send PASS command */
-	  write (conn->socket, r_pass, strlen (r_pass));
-	  while ((nb_read = read (conn->socket, buffer, UC_BUFFER_LEN)) > 0)
-	    {
-	      buffer[nb_read] = '\0';
-	      if (strstr (buffer, "230 "))
-		break;
-	      else if (strstr (buffer, UC_STATUS_CODE_FTP_MAX_CLIENTS))
-		{
-		  g_array_append_vals (conn->array,
-				       UC_STATUS_CODE_FTP_MAX_CLIENTS,
-				       strlen
-				       (UC_STATUS_CODE_FTP_MAX_CLIENTS));
-		  goto handler_exit;
-		}
-	    }
-	}
-      else
-	{
-	  g_warning ("ftp: problem while reading after USER command (%s)",
-		     conn->hostname);
-	  goto handler_error;
-	}
-
-      /* we first test with CWD. if this is a directory and if it exist,
-       * no need to request go further: it is ok */
-      request = g_strdup_printf ("CWD %s\n", conn->request);
-      write (conn->socket, request, strlen (request));
-      sleep (1);
-      g_free (request), request = NULL;
-
-      nb_read = read (conn->socket, buffer, UC_BUFFER_LEN);
-      if (uc_utils_ftp_code_search (buffer, "250", nb_read))
-	{
-	  g_array_append_vals (conn->array, UC_STATUS_CODE_FTP_OK,
-			       strlen (UC_STATUS_CODE_FTP_OK));
-	  goto handler_exit;
-	}
-
-      /* send PASV command */
-      write (conn->socket, r_pasv, strlen (r_pasv));
-      sleep (1);
-
-      nb_read = read (conn->socket, buffer, UC_BUFFER_LEN);
       buffer[nb_read] = '\0';
-
-      if (!uc_utils_ftp_code_search (buffer, "227", nb_read) || nb_read <= 4)
-	{
-	  g_warning ("FTP: problem with passive mode (%s)", conn->hostname);
-	  goto handler_error;
-	}
-
-      buffer[nb_read - 3] = '\0';
-      if ((port = strchr (buffer, '(')) != NULL)
-	port++;
-      else
-	{
-	  g_warning ("FTP: problem reading passive mode line (%s)",
-		     conn->hostname);
-	  goto handler_error;
-	}
-
-      if (sscanf (port, "%u,%u,%u,%u,%u,%u",
-		  &temp[0], &temp[1], &temp[2], &temp[3], &temp[4],
-		  &temp[5]) != 6)
-	{
-	  g_warning ("FTP: problem building passive port (%s)",
-		     conn->hostname);
-	  goto handler_error;
-	}
-
-      request = g_strdup_printf ("LIST %s\n", conn->request);
-
-      write (conn->socket, request, strlen (request));
-      g_free (request), request = NULL;
-
-      /* FTP passive connection */
-      conn->ftp_passive_port = (temp[4] * 256) + temp[5];
-      conn->ftp_passive = TRUE;
-      conn->thread_ftp_end = FALSE;
-      pthread_create (&thread, NULL, connect_thread, conn);
-      pthread_detach (thread);
-
-      UC_THREAD_WAIT (conn->thread_ftp_end, FALSE);
-      pthread_cancel (thread);
-      conn->thread_ftp_end = FALSE;
-      conn->ftp_passive = FALSE;
-
-      if (conn->tcp_status != UC_TCP_STATUS_OK)
-	{
-	  g_warning
-	    ("FTP: problem while connecting on passive port (%s)",
-	     conn->hostname);
-	  goto handler_error;
-	}
-      else
-	{
-	  gchar *state = NULL;
-
-	  while ((nb_read =
-		  read (conn->ftp_passive_socket, buffer, UC_BUFFER_LEN)) > 0)
-	    have_data = TRUE;
-
-	  shutdown (conn->ftp_passive_socket, SHUT_RDWR);
-	  pthread_cancel (thread);
-	  close (conn->ftp_passive_socket);
-
-	  state = (have_data) ? UC_STATUS_CODE_FTP_OK : UC_STATUS_CODE_FTP_OK;
-	  g_array_append_vals (conn->array, state, strlen (state));
-
-	  /* send QUIT command */
-	  write (conn->socket, r_quit, strlen (r_quit));
-	}
-
-      goto handler_exit;
+      if (strstr (buffer, "220 "))
+        break;
     }
-  /* HTTP */
-  else if (conn->socket > 0)
+
+    // Send USER command
+    write (conn->socket, r_user, strlen (r_user));
+    // FIXME g_usleep (G_USEC_PER_SEC);
+    nb_read = read (conn->socket, buffer, UC_BUFFER_LEN);
+
+    // 230: User logged in, proceed
+    if (uc_utils_ftp_code_search (buffer, "230", nb_read))
+      ;
+    // 332: Need account for login
+    else if (uc_utils_ftp_code_search (buffer, "332", nb_read))
     {
-      write (conn->socket, conn->request, strlen (conn->request));
-      while (!tmp &&
-	     (nb_read = read (conn->socket, buffer, UC_BUFFER_LEN)) > 0)
-	{
-	  if (conn->header_request)
-	    {
-	      gchar b = buffer[nb_read];
-	      buffer[nb_read] = '\0';
-	      if ((tmp = strstr (buffer, "\r\n\r\n")))
-		{
-		  *tmp = '\0';
-		  g_array_append_vals (conn->array, buffer, strlen (buffer));
-		  nb_read = 0;
-		}
-	      else
-		buffer[nb_read] = b;
-	    }
-
-	  if (nb_read > 0)
-	    g_array_append_vals (conn->array, buffer, nb_read);
-	}
-
-      conn->tcp_status =
-	(nb_read == 0) ? UC_TCP_STATUS_OK : UC_TCP_STATUS_ERROR;
-
-      goto handler_exit;
+      g_warning ("FTP: Need account for login (%s)", conn->hostname);
+      goto handler_error;
     }
+    // 331: User name okay, need password
+    else if (uc_utils_ftp_code_search (buffer, "331", nb_read))
+    {
+      // Send PASS command
+      write (conn->socket, r_pass, strlen (r_pass));
+      while ((nb_read = read (conn->socket, buffer, UC_BUFFER_LEN)) > 0)
+      {
+        buffer[nb_read] = '\0';
+        if (strstr (buffer, "230 "))
+          break;
+        else if (strstr (buffer, UC_STATUS_CODE_FTP_MAX_CLIENTS) != NULL)
+        {
+          g_array_append_vals (conn->array, UC_STATUS_CODE_FTP_MAX_CLIENTS,
+                               strlen (UC_STATUS_CODE_FTP_MAX_CLIENTS));
+          goto handler_exit;
+        }
+      }
+    }
+    else
+    {
+       g_warning ("ftp: problem while reading after USER command (%s)",
+                  conn->hostname);
+       goto handler_error;
+    }
+
+    // We first test with CWD. if this is a directory and if it exist,
+    // no need to request go further: it is ok
+    request = g_strdup_printf ("CWD %s\n", conn->request);
+    write (conn->socket, request, strlen (request));
+    // FIXME g_usleep (G_USEC_PER_SEC);
+    g_free (request), request = NULL;
+
+    nb_read = read (conn->socket, buffer, UC_BUFFER_LEN);
+    if (uc_utils_ftp_code_search (buffer, "250", nb_read))
+    {
+       g_array_append_vals (conn->array, UC_STATUS_CODE_FTP_OK,
+                            strlen (UC_STATUS_CODE_FTP_OK));
+       goto handler_exit;
+    }
+
+    // Send PASV command
+    write (conn->socket, r_pasv, strlen (r_pasv));
+    // FIXME g_usleep (G_USEC_PER_SEC); 
+    nb_read = read (conn->socket, buffer, UC_BUFFER_LEN);
+    buffer[nb_read] = '\0';
+
+    if (!uc_utils_ftp_code_search (buffer, "227", nb_read) || nb_read <= 4)
+    {
+      g_warning ("FTP: problem with passive mode (%s)", conn->hostname);
+      goto handler_error;
+    }
+
+    buffer[nb_read - 3] = '\0';
+    if ((port = strchr (buffer, '(')) != NULL)
+      port++;
+    else
+    {
+      g_warning ("FTP: problem reading passive mode line (%s)", conn->hostname);
+      goto handler_error;
+    }
+
+    if (sscanf (port, "%u,%u,%u,%u,%u,%u",
+                &temp[0], &temp[1], &temp[2], &temp[3], &temp[4],
+                &temp[5]) != 6)
+    {
+      g_warning ("FTP: problem building passive port (%s)", conn->hostname);
+      goto handler_error;
+    }
+
+    request = g_strdup_printf ("LIST %s\n", conn->request);
+
+    write (conn->socket, request, strlen (request));
+    g_free (request), request = NULL;
+
+    // FTP passive connection
+    conn->ftp_passive_port = (temp[4] * 256) + temp[5];
+    conn->ftp_passive = TRUE;
+    conn->thread_ftp_end = FALSE;
+    pthread_create (&thread, NULL, connect_thread, conn);
+    pthread_detach (thread);
+
+    UC_THREAD_WAIT (conn->thread_ftp_end, FALSE);
+    pthread_cancel (thread);
+    conn->thread_ftp_end = FALSE;
+    conn->ftp_passive = FALSE;
+
+    if (conn->tcp_status != UC_TCP_STATUS_OK)
+    {
+      g_warning ("FTP: problem while connecting on passive port (%s)",
+                 conn->hostname);
+       goto handler_error;
+    }
+    else
+    {
+      gchar *state = NULL;
+
+
+       while ((nb_read =
+                 read (conn->ftp_passive_socket, buffer, UC_BUFFER_LEN)) > 0)
+         have_data = TRUE;
+
+       shutdown (conn->ftp_passive_socket, SHUT_RDWR);
+       pthread_cancel (thread);
+       close (conn->ftp_passive_socket);
+
+       state = (have_data) ? UC_STATUS_CODE_FTP_OK : UC_STATUS_CODE_FTP_OK;
+       g_array_append_vals (conn->array, state, strlen (state));
+
+       // Send QUIT command
+       write (conn->socket, r_quit, strlen (r_quit));
+    }
+
+    goto handler_exit;
+  }
+  // HTTP
+  else if (conn->socket > 0)
+  {
+    write (conn->socket, conn->request, strlen (conn->request));
+    while (!tmp && (nb_read = read (conn->socket, buffer, UC_BUFFER_LEN)) > 0)
+    {
+      if (conn->header_request)
+      {
+        gchar b = buffer[nb_read];
+
+
+        buffer[nb_read] = '\0';
+        if ((tmp = strstr (buffer, "\r\n\r\n")))
+        {
+          *tmp = '\0';
+          g_array_append_vals (conn->array, buffer, strlen (buffer));
+          nb_read = 0;
+        }
+        else
+          buffer[nb_read] = b;
+      }
+
+      if (nb_read > 0)
+        g_array_append_vals (conn->array, buffer, nb_read);
+    }
+
+    conn->tcp_status = (nb_read == 0) ? UC_TCP_STATUS_OK : UC_TCP_STATUS_ERROR;
+
+    goto handler_exit;
+  }
 
 handler_error:
   conn->tcp_status = UC_TCP_STATUS_ERROR;
@@ -609,6 +614,7 @@ handler_exit:
   pthread_exit (NULL);
 }
 
+
 /**
  * connect_nonblock:
  * @soc: Socket to connect.
@@ -619,7 +625,7 @@ handler_exit:
  * Returns: #UCTcpStatus of the operation.
  */
 static UCTcpStatus
-connect_nonblock (const gint soc, struct sockaddr_in addr)
+connect_nonblock (const gint soc, struct sockaddr *addr, socklen_t addrlen)
 {
   gint res;
   glong arg;
@@ -628,6 +634,7 @@ connect_nonblock (const gint soc, struct sockaddr_in addr)
   gint valopt;
   socklen_t lon;
 
+
   if ((arg = fcntl (soc, F_GETFL, NULL)) < 0)
     {
       /* g_warning ("Error fcntl(..., F_GETFL) (%s)\n", strerror (errno)); */
@@ -641,7 +648,7 @@ connect_nonblock (const gint soc, struct sockaddr_in addr)
       return UC_TCP_STATUS_ERROR;
     }
 
-  res = connect (soc, (struct sockaddr *) &addr, sizeof (addr));
+  res = connect (soc, addr, addrlen);
   if (res < 0)
     {
       if (errno == EINPROGRESS)
@@ -711,6 +718,7 @@ connect_nonblock (const gint soc, struct sockaddr_in addr)
   return UC_TCP_STATUS_OK;
 }
 
+
 /**
  * connect_thread:
  * @data: a #UCConn object.
@@ -723,72 +731,75 @@ static void *
 connect_thread (void *data)
 {
   UCConn *conn = (UCConn *) data;
-  struct hostent *hostent;
-  struct sockaddr_in addr;
+  struct addrinfo hints; 
+  struct addrinfo *res = NULL; 
   gint *sock = NULL;
-  guint *port = NULL;
+  gchar *port = NULL;
+  gint ret = 0;
+
 
   if (conn->ftp_passive)
-    {
-      sock = &conn->ftp_passive_socket;
-      port = &conn->ftp_passive_port;
-    }
+  {
+    sock = &conn->ftp_passive_socket;
+    port = g_strdup_printf ("%u", conn->ftp_passive_port);
+  }
   else
-    {
-      sock = &conn->socket;
-      port = &conn->port;
-    }
+  {
+    sock = &conn->socket;
+    port = g_strdup_printf ("%u", conn->port);
+  }
 
-  /* resolve hostname (ok if IP) */
-  hostent = gethostbyname ((const char *) conn->hostname);
-  if (!hostent)
-    {
-      /* FIXME conn->tcp_status = UC_TCP_STATUS_BAD_HOSTNAME; */
-      g_warning ("Problem with gethostbyname: %s:%u", conn->hostname, *port);
-      conn->tcp_status = UC_TCP_STATUS_ERROR;
-    }
-  else
-    {
-      memset (&addr, 0, sizeof (struct sockaddr_in));
-      addr.sin_port = htons (*port);
-      addr.sin_family = AF_INET;
-      memcpy (&addr.sin_addr, hostent->h_addr_list[0], hostent->h_length);
+  // Free port at thread exit
+  pthread_cleanup_push (g_free, port);
 
-      /* create socket */
-      if ((*sock = socket (AF_INET, SOCK_STREAM, 0)) < 0)
-	{
-	  g_warning ("Problem with socket creation: %s:%u", conn->hostname,
-		     *port);
-	  conn->tcp_status = UC_TCP_STATUS_ERROR;
-	}
+  memset (&hints, 0, sizeof (hints));
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_family = PF_UNSPEC;
 
-      /* connect socket */
-      else if ((conn->tcp_status =
-		connect_nonblock (*sock, addr)) == UC_TCP_STATUS_OK)
-	{
+  if ((ret = getaddrinfo (conn->hostname, port, &hints, &res)) != 0 ||
+      res == NULL)
+  {
+    /* FIXME conn->tcp_status = UC_TCP_STATUS_BAD_HOSTNAME; */
+    g_warning ("Problem with getaddrinfo(): %s:%s\n\t-> %s",
+               conn->hostname, port, gai_strerror (ret));
+    conn->tcp_status = UC_TCP_STATUS_ERROR;
+  }
+  // TODO check all returned addresses for the hostname
+  else if ((*sock = socket (res->ai_family, res->ai_socktype,
+                            res->ai_protocol)) >= 0)
+  {
+    // Free getaddrinfo result at thread exit
+    pthread_cleanup_push ((void *) freeaddrinfo, res);
+
+    /* connect socket */
+    if ((conn->tcp_status =
+           connect_nonblock (*sock, res->ai_addr,
+                             res->ai_addrlen)) == UC_TCP_STATUS_OK)
+    {
 #ifdef ENABLE_GNUTLS
-	  if (conn->use_gnutls)
-	    {
-	      gint ret = 0;
+      if (conn->use_gnutls)
+      {
+        gnutls_transport_set_ptr (conn->session_gnutls,
+                                   (gnutls_transport_ptr_t) conn->socket);
+        do
+          ret = gnutls_handshake (conn->session_gnutls);
+        while ((ret == GNUTLS_E_AGAIN) || (ret == GNUTLS_E_INTERRUPTED));
 
-	      gnutls_transport_set_ptr (conn->session_gnutls,
-					(gnutls_transport_ptr) conn->socket);
-	      do
-		ret = gnutls_handshake (conn->session_gnutls);
-	      while ((ret == GNUTLS_E_AGAIN)
-		     || (ret == GNUTLS_E_INTERRUPTED));
-
-	      if (ret < 0)
-		{
-		  g_warning ("Problem with GNU TLS handshake: %s:%u",
-			     conn->hostname, *port);
-		  conn->tcp_status = UC_TCP_STATUS_ERROR;
-		}
-	    }
+        if (ret < 0)
+        {
+          g_warning ("Problem with SSL handshake: %s:%s", conn->hostname, port);
+          gnutls_perror (ret);
+          conn->tcp_status = UC_TCP_STATUS_ERROR;
+        }
+      }
 #endif
-	}
     }
 
+    pthread_cleanup_pop (1); // freeaddrinfo (res)
+  }
+
+  pthread_cleanup_pop (1); // g_free (port)
+
   if (conn->ftp_passive)
     conn->thread_ftp_end = TRUE;
   else
