@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -62,13 +62,20 @@
 #include <buffio.h>
 #endif
 #endif
+#ifdef ENABLE_SQLITE3
+#include <sqlite3.h>
+#endif
+#ifdef ENABLE_JSON
+#include <json-glib/json-glib.h>
+#endif
+
 
 /**
  * UC_SYSTEM_NAME:
  *
  * Macro to determine the system we are running on.
  */
-#ifdef __linux__
+#if defined(__linux__) || defined(__GLIBC__)
 #define UC_SYSTEM_NAME "Linux"
 #include <sys/vfs.h>
 #elif defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD__)
@@ -99,8 +106,6 @@
 
 #define UC_VERSION VERSION
 
-#define UC_ARRAY_SIZE(array) (sizeof (array) / sizeof (*(array)))
-
 #define UC_UPDATE_UI \
 	while (g_main_context_pending (NULL)) \
 	{ \
@@ -174,48 +179,6 @@
  */
 #define UC_CHECK_MESSAGE_LABEL_THIRD \
 	2
-/**
- * UC_CHECK_DISPLAY_GOODLINKS:
- *
- * Macro to indicate that we must display only good links.
- */
-#define UC_CHECK_DISPLAY_GOODLINKS \
-	0
-/**
- * UC_CHECK_DISPLAY_BADLINKS:
- *
- * Macro to indicate that we must display only bad links.
- */
-#define UC_CHECK_DISPLAY_BADLINKS \
-	1
-/**
- * UC_CHECK_DISPLAY_TIMEOUTS:
- *
- * Macro to indicate that we must display only timeouts links.
- */
-#define UC_CHECK_DISPLAY_TIMEOUTS \
-	2
-/**
- *  UC_CHECK_DISPLAY_MALFORMEDLINKS:
- *
- *  Macro to indicate that we must display only malformed links.
- */
-#define UC_CHECK_DISPLAY_MALFORMEDLINKS \
-	3
-/**
- *  UC_CHECK_DISPLAY_SECURITY_ALERTS:
- *
- *  Macro to indicate that we must display only links with security alert.
- */
-#define UC_CHECK_DISPLAY_SECURITY_ALERTS \
-	4
-/**
- *  UC_CHECK_DISPLAY_W3C_ALERTS:
- *
- *  Macro to indicate that we must display only links with w3c validity alert.
- */
-#define UC_CHECK_DISPLAY_W3C_ALERTS \
-	5
 
 /**
  * UC_STATUS_CODE_INTERNALS_LIMIT:
@@ -493,43 +456,6 @@
 
 #define UC_SPLIT_DELIMITER \
 	","
-/**
- * UC_DEFAULT_DOCUMENTS:
- *
- * Macro to indicate which documents extensions we want to manage.
- */
-#define UC_DEFAULT_DOCUMENTS \
-	"avi,mov," \
-	"ogg,mp3,wav,midi,ram," \
-        "cfm,html,htm,pl,phtml,shtml,asp,axp,php,php3,php4,php5,xml,rdf,xsl," \
-	"css," \
-	"pdf,doc,xls,ppt,swf,sxw,sxc,sxi," \
-	"tar,gz,tgz,zip,bz2,rar," \
-	"deb,rpm," \
-	"pif,exe,com,sys"
-/**
- * UC_DEFAULT_DOCUMENTS_LEN:
- *
- * Macro to indicate the number of documents extensions we want to
- * manage.
- */
-#define UC_DEFAULT_DOCUMENTS_LEN \
-	43
-/**
- * UC_DEFAULT_IMAGES:
- *
- * Macro to indicate which images extensions we want to manage.
- */
-#define UC_DEFAULT_IMAGES \
-	"png,gif,jpeg,xpm,bmp,jpg,tiff,ico"
-/**
- * UC_DEFAULT_IMAGES_LEN:
- *
- * Macro to indicate the number of images extensions we want to
- * manage.
- */
-#define UC_DEFAULT_IMAGES_LEN \
-	8
 
 /**
  * UC_DEFAULT_SECURITY_BAD_EXTENSIONS:
@@ -538,14 +464,6 @@
  */
 #define UC_DEFAULT_SECURITY_BAD_EXTENSIONS \
 	"bak,bad,sav,~,ok,-,old,orig,false"
-/**
- * UC_DEFAULT_SECURITY_BAD_EXTENSIONS_LEN:
- *
- * Macro to indicate the number of bad extensions we want to
- * manage.
- */
-#define UC_DEFAULT_SECURITY_BAD_EXTENSIONS_LEN \
-	9
 
 /**
  * UC_DEFAULT_SECURITY_VIRII_EXTENSIONS:
@@ -556,30 +474,14 @@
 	"application,audio,image,text,video," \
 	"chemical,model,multipart,x-conference,x-epoc,x-world"
 
-/**
- * UC_DEFAULT_SECURITY_VIRII_EXTENSIONS_LEN:
- *
- * Macro to indicate the number of extensions we want to
- * manage for virii.
- */
-#define UC_DEFAULT_SECURITY_VIRII_EXTENSIONS_LEN \
-	11
-
 /**
  * UC_DEFAULT_W3C_HTML_EXTENSIONS:
  *
  * Macro to indicate for which extensions we must search for HTML validation.
  */
 #define UC_DEFAULT_W3C_HTML_EXTENSIONS \
-        "cfm,html,htm,pl,phtml,shtml,asp,axp,php,php3,php4,php5"
-/**
- * UC_DEFAULT_W3C_HTML_EXTENSIONS_LEN:
- *
- * Macro to indicate the number of extensions we want to
- * manage for HTML validation.
- */
-#define UC_DEFAULT_W3C_HTML_EXTENSIONS_LEN \
-	12
+        "cfm,html,htm,pl,phtml,shtml,asp,axp,aspx,php,php3,php4,php5"
+
 /**
  * UC_DEFAULT_W3C_CSS_EXTENSIONS:
  *
@@ -587,14 +489,6 @@
  */
 #define UC_DEFAULT_W3C_CSS_EXTENSIONS \
         "css"
-/**
- * UC_DEFAULT_W3C_CSS_EXTENSIONS_LEN:
- *
- * Macro to indicate the number of extensions we want to
- * manage for HTML validation.
- */
-#define UC_DEFAULT_W3C_CSS_EXTENSIONS_LEN \
-	1
 
 /**
  * UC_LABEL_DISPLAY_MAX_LEN:
@@ -605,72 +499,6 @@
  */
 #define UC_LABEL_DISPLAY_MAX_LEN \
 	45
-/**
- * UC_GET_DEFAULT_DOCUMENTS:
- *
- * Macro to get documents extensions we can manage.
- */
-#define UC_GET_DEFAULT_DOCUMENTS \
-	g_strsplit( \
-		UC_DEFAULT_DOCUMENTS, \
-		UC_SPLIT_DELIMITER, \
-		UC_DEFAULT_DOCUMENTS_LEN \
-        )
-/**
- * UC_GET_DEFAULT_IMAGES:
- *
- * Macro to get images extensions we can manage.
- */
-#define UC_GET_DEFAULT_IMAGES \
-	g_strsplit( \
-		UC_DEFAULT_IMAGES, \
-		UC_SPLIT_DELIMITER, \
-		UC_DEFAULT_IMAGES_LEN \
-        )
-/**
- * UC_GET_DEFAULT_SECURITY_BAD_EXTENSIONS:
- *
- * Macro to get bad extensions we can manage.
- */
-#define UC_GET_DEFAULT_SECURITY_BAD_EXTENSIONS \
-	g_strsplit( \
-		UC_DEFAULT_SECURITY_BAD_EXTENSIONS, \
-		UC_SPLIT_DELIMITER, \
-		UC_DEFAULT_SECURITY_BAD_EXTENSIONS_LEN \
-        )
-/**
- * UC_GET_DEFAULT_SECURITY_VIRII_EXTENSIONS:
- *
- * Macro to get extensions we can manage for virii.
- */
-#define UC_GET_DEFAULT_SECURITY_VIRII_EXTENSIONS \
-	g_strsplit( \
-		UC_DEFAULT_SECURITY_VIRII_EXTENSIONS, \
-		UC_SPLIT_DELIMITER, \
-		UC_DEFAULT_SECURITY_VIRII_EXTENSIONS_LEN \
-        )
-/**
- * UC_GET_DEFAULT_W3C_HTML_EXTENSIONS:
- *
- * Macro to get extensions we can manage for HTML validation.
- */
-#define UC_GET_DEFAULT_W3C_HTML_EXTENSIONS \
-	g_strsplit( \
-		UC_DEFAULT_W3C_HTML_EXTENSIONS, \
-		UC_SPLIT_DELIMITER, \
-		UC_DEFAULT_W3C_HTML_EXTENSIONS_LEN \
-        )
-/**
- * UC_GET_DEFAULT_W3C_CSS_EXTENSIONS:
- *
- * Macro to get extensions we can manage for CSS validation.
- */
-#define UC_GET_DEFAULT_W3C_CSS_EXTENSIONS \
-	g_strsplit( \
-		UC_DEFAULT_W3C_CSS_EXTENSIONS, \
-		UC_SPLIT_DELIMITER, \
-		UC_DEFAULT_W3C_CSS_EXTENSIONS_LEN \
-        )
 
 /**
  * UC_DEFAULT_WEB_BROWSER_PATH:
@@ -712,7 +540,7 @@
  */
 #define UC_SET_WIDGET(name, window, widget) \
 	g_object_set_data_full (G_OBJECT (window), name, \
-	gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)
+	g_object_ref (widget), (GDestroyNotify) g_object_unref)
 /**
  * WGET:
  * @name: name of the widget to retreive.
@@ -723,10 +551,6 @@
 #define WGET(name) \
 	glade_xml_get_widget (glade, name)
 
-#define UC_LABEL_DISPLAY_IN_TABLE(table, label, name, p1, p2, p3, p4) \
-        gtk_label_set_markup (GTK_LABEL (label), name); \
-        gtk_table_attach (GTK_TABLE (table), label, p1, p2, p3, p4, \
-		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0)
 /**
  * UC_BUFFER_LEN:
  *
@@ -909,21 +733,6 @@
 	TRUE
 #define UC_SOCKET_OPEN_DEFAULT \
 	FALSE
-/**
- * UC_SECURE_BYTES_ADDING:
- * 
- * Macro to indicate security adding for free partition space verification for
- * the working directory.
- */
-#define UC_SECURE_BYTES_ADDING \
-	5000000
-/**
- * UC_CHECK_MINIMUM_SPACE_REQUIRED:
- *
- * Macro to indicate the minimum free space we must leave on the disk.
- */
-#define UC_CHECK_MINIMUM_SPACE_REQUIRED \
-	10000000
 /**
  * UC_TOOLTIPS_DELAY_DEFAULT:
  *
@@ -1046,6 +855,71 @@ struct _UCStatusCode
   GdkPixbuf *icon_file;
 };
 
+#define UC_DISPLAY_PROTO_SET_NONE \
+  current_display_proto = UC_DISPLAY_PROTO_NONE
+
+#define UC_DISPLAY_STATUS_SET_NONE \
+  current_display_status = UC_DISPLAY_STATUS_NONE
+
+#define UC_DISPLAY_TYPE_SET_NONE \
+  current_display_type = UC_DISPLAY_TYPE_NONE
+
+#define UC_DISPLAY_SET_ALL \
+  UC_DISPLAY_TYPE_SET_NONE;\
+  UC_DISPLAY_STATUS_SET_NONE; \
+  UC_DISPLAY_PROTO_SET_NONE
+
+#define UC_DISPLAY_PROTO_SET(flag) current_display_proto |= (flag)
+#define UC_DISPLAY_PROTO_UNSET(flag) current_display_proto &= ~(flag)
+#define UC_DISPLAY_PROTO_IS(flag) (current_display_proto & flag)
+#define UC_DISPLAY_PROTO_IS_NONE (current_display_proto == 0)
+
+typedef enum _UCDisplayProto UCDisplayProto;
+enum _UCDisplayProto
+{
+  UC_DISPLAY_PROTO_NONE = 0,
+  UC_DISPLAY_PROTO_HTTP = 1,
+  UC_DISPLAY_PROTO_HTTPS  = 1 << 1,
+  UC_DISPLAY_PROTO_FTP = 1 << 2,
+  UC_DISPLAY_PROTO_FILE = 1 << 3
+};
+UCDisplayProto current_display_proto;
+
+#define UC_DISPLAY_STATUS_SET(flag) current_display_status |= (flag)
+#define UC_DISPLAY_STATUS_UNSET(flag) current_display_status &= ~(flag)
+#define UC_DISPLAY_STATUS_IS(flag) (current_display_status & flag)
+#define UC_DISPLAY_STATUS_IS_NONE (current_display_status == 0)
+
+typedef enum _UCDisplayStatus UCDisplayStatus;
+enum _UCDisplayStatus
+{
+  UC_DISPLAY_STATUS_NONE = 0,
+  UC_DISPLAY_STATUS_GOODLINKS = 1,
+  UC_DISPLAY_STATUS_BADLINKS  = 1 << 1,
+  UC_DISPLAY_STATUS_TIMEOUTS = 1 << 2,
+  UC_DISPLAY_STATUS_MALFORMEDLINKS = 1 << 3,
+  UC_DISPLAY_STATUS_SECURITY_ALERTS = 1 << 4,
+  UC_DISPLAY_STATUS_W3C_ALERTS = 1 << 5
+};
+UCDisplayStatus current_display_status;
+
+#define UC_DISPLAY_TYPE_SET(flag) current_display_type |= (flag)
+#define UC_DISPLAY_TYPE_UNSET(flag) current_display_type &= ~(flag)
+#define UC_DISPLAY_TYPE_IS(flag) (current_display_type & flag)
+#define UC_DISPLAY_TYPE_IS_NONE (current_display_type == 0)
+
+typedef enum _UCDisplayType UCDisplayType;
+enum _UCDisplayType
+{
+  UC_DISPLAY_TYPE_NONE = 0,
+  UC_DISPLAY_TYPE_HREF = 1,
+  UC_DISPLAY_TYPE_IMAGE = 1 << 1,
+  UC_DISPLAY_TYPE_FRAME = 1 << 2,
+  UC_DISPLAY_TYPE_EMAIL = 1 << 3,
+  UC_DISPLAY_TYPE_CSS = 1 << 4
+};
+UCDisplayType current_display_type;
+
 /**
  * UCLinkType:
  * @LINK_TYPE_NONE: No specific type.
@@ -1065,7 +939,6 @@ struct _UCStatusCode
  * @LINK_TYPE_FRAME: FRAME or IFRAME tag (HTTP).
  * @LINK_TYPE_FRAME_SSL: FRAME or IFRAME tag (HTTPS).
  * @LINK_TYPE_BOOKMARK_FOLDER: A bookmark folder.
- * @LINK_TYPES_LEN: Length of the #UCLinkType structure.
  * @LINK_TYPES_SECURITY_TEST_BAD_EXTENSIONS: A fake link to test existence of 
  *                                           bad files on the server.
  * 
@@ -1091,8 +964,26 @@ enum _UCLinkType
   LINK_TYPE_FRAME,
   LINK_TYPE_FRAME_SSL,
   LINK_TYPE_BOOKMARK_FOLDER,
-  LINK_TYPES_SECURITY_TEST_BAD_EXTENSIONS,
-  LINK_TYPES_LEN
+  LINK_TYPES_SECURITY_TEST_BAD_EXTENSIONS
+};
+
+/**
+ * UCBookmarksType:
+ * @UC_BOOKMARKS_TYPE_XBEL: xbel file.
+ * @UX_BOOKMARKS_TYPE_FIREFOX: Firefox sqlite3 file.
+ * @UX_BOOKMARKS_TYPE_GOOGLECHROME: Google Chrome json file.
+ * @UX_BOOKMARKS_TYPE_OPERA: Opera file.
+ *
+ * Type of current bookmark file.
+ */
+typedef enum _UCBookmarksType UCBookmarksType;
+enum _UCBookmarksType
+{
+  UC_BOOKMARKS_TYPE_NONE,
+  UC_BOOKMARKS_TYPE_XBEL,
+  UC_BOOKMARKS_TYPE_FIREFOX,
+  UC_BOOKMARKS_TYPE_GOOGLECHROME,
+  UC_BOOKMARKS_TYPE_OPERA
 };
 
 /**
@@ -1120,7 +1011,10 @@ enum _UCEmailStatus
 typedef enum _UCExportFormat UCExportFormat;
 enum _UCExportFormat
 {
-  UC_EXPORT_HTML
+  UC_EXPORT_HTML,
+  UC_EXPORT_CSV,
+  UC_EXPORT_LIST_NORMAL,
+  UC_EXPORT_LIST_SIMPLE
 };
 
 typedef enum _UCCookiesAction UCCookiesAction;
@@ -1180,6 +1074,9 @@ struct _UCHTMLTag
  * @checked: TRUE is this link has already been proccessed.
  * @xml_node: Reference on the XmlNodePtr of the XmlDocPtr for this node. This
  *            field is used only for bookmarks.
+ * @bookmark_folder_id: Reference on the parent id. This field is used only for FF
+ *                      sqlite3 bookmarks.
+ * @bookmark_id: id of the bookmark in FF database
  * @to_delete: TRUE if this node has been deleted. This field is used only for
  *             bookmarks.
  * @metas: #GList of page HTML meta tags (pointers on this node "tags"
@@ -1194,6 +1091,10 @@ struct _UCHTMLTag
  * @bad_extensions: #GList of bad extensions that exists on the server for 
  *                  this node.
  * @tags: #GList of #UCHTMLTag for all links in this page.
+ * @user_action: A user defined action label
+ * @virii: if the file is a virus or not 
+ * @virname: Name of the virus is @virii == %TRUE
+ * @w3c_valid: %TRUE if the document is in a correct format
  * 
  * Structure to store links properties.
  */
@@ -1227,6 +1128,11 @@ struct _UCLinkProperties
   gboolean is_downloadable;
   gboolean checked;
   xmlNodePtr xml_node;
+#ifdef ENABLE_JSON
+  JsonArray *json_array;
+#endif
+  guint32 bookmark_folder_id;
+  guint32 bookmark_id;
   gboolean to_delete;
   GList *metas;
   GList *emails;
