@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2002-2005 
+ * Copyright (C) 2002-2010
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,11 @@
 #include "utils.h"
 #include "url.h"
 #include "html_parser.h"
+#include "project.h"
 
 #include "lists.h"
 
+
 /*
  * struct used for the uc_lists_already_checked_links_list
  * list
@@ -45,18 +47,13 @@ static GList *uc_lists_checked_links_list = NULL;
  * a update */
 static GList *uc_lists_refresh_preserved_links_list = NULL;
 
-static GList *uc_lists_checked_links_free_real (GList * list);
-static UCAlreadyChecked *uc_lists_already_checked_links_node_new (const
-								  gchar *
-								  url,
-								  gpointer
-								  ac_p);
-static UCLinkProperties *uc_lists_checked_links_lookup_by_uid_real (GList *
-								    list,
-								    const
-								    guint
-								    uid);
-static GList *uc_lists_html_tags_node_free (GList * list, UCHTMLTag * tag);
+static void uc_lists_checked_links_free_real (GList **list);
+static UCAlreadyChecked *uc_lists_already_checked_links_node_new (
+  const gchar *url, gpointer ac_p);
+static UCLinkProperties *uc_lists_checked_links_lookup_by_uid_real (
+  GList *list, const guint uid);
+static void uc_lists_html_tags_free (GList ** list);
+
 
 void
 uc_lists_refresh_preserved_links_reset (void)
@@ -82,38 +79,37 @@ uc_lists_refresh_preserved_links_restore (GList * src)
 /*
  * free a html tags list node
  */
-static GList *
-uc_lists_html_tags_node_free (GList * list, UCHTMLTag * tag)
+void
+uc_lists_html_tags_node_free (GList **list, UCHTMLTag **tag)
 {
-  list = g_list_remove (list, (gpointer) tag);
-  uc_html_parser_node_free (tag), tag = NULL;
-
-  return list;
+  *list = g_list_remove (*list, (gpointer) *tag);
+  uc_html_parser_node_free (tag);
 }
 
+
 /*
  * free a html tags list
  */
-void
-uc_lists_html_tags_free (GList * list)
+static void
+uc_lists_html_tags_free (GList ** list)
 {
   GList *item = NULL;
 
-  g_return_if_fail (list != NULL);
 
-  item = g_list_first (list);
+  g_return_if_fail (*list != NULL);
+
+  item = g_list_first (*list);
   while (item != NULL)
-    {
-      UCHTMLTag *tag = (UCHTMLTag *) item->data;
+  {
+    UCHTMLTag *tag = (UCHTMLTag *) item->data;
+    item = g_list_next (item);
+    uc_lists_html_tags_node_free (list, &tag);
+  }
 
-      item = g_list_next (item);
-
-      list = uc_lists_html_tags_node_free (list, tag);
-    }
-
-  g_list_free (list), list = NULL;
+  g_list_free (*list), *list = NULL;
 }
 
+
 /*
  * append a item to the uc_lists_already_checked_links_list
  * list
@@ -152,101 +148,97 @@ uc_lists_checked_links_is_empty (void)
 void
 uc_lists_checked_links_free (void)
 {
-  uc_lists_checked_links_list =
-    uc_lists_checked_links_free_real (uc_lists_checked_links_list),
-    uc_lists_checked_links_list = NULL;
+  uc_lists_checked_links_free_real (&uc_lists_checked_links_list);
 }
 
+
 /*
  * free a UCLinkProperties node
  */
-GList *
-uc_lists_checked_links_node_free (GList * list, UCLinkProperties * prop)
+void
+uc_lists_checked_links_node_free (GList ** list, UCLinkProperties ** prop)
 {
-  if (prop == NULL)
-    return list;
+  if (*prop == NULL)
+    return;
 
-  g_free (prop->treeview_path), prop->treeview_path = NULL;
-  if (prop->link_icon != NULL)
-    g_object_unref (prop->link_icon), prop->link_icon = NULL;
-  g_free (prop->user_action), prop->user_action = NULL;
-  g_free (prop->link_value), prop->link_value = NULL;
-  g_free (prop->label), prop->label = NULL;
-  g_free (prop->url), prop->url = NULL;
-  g_free (prop->normalized_url), prop->normalized_url = NULL;
-  g_free (prop->protocol), prop->protocol = NULL;
-  g_free (prop->h_name), prop->h_name = NULL;
-  g_free (prop->port), prop->port = NULL;
-  g_free (prop->path), prop->path = NULL;
-  g_free (prop->domain), prop->domain = NULL;
-  g_free (prop->args), prop->args = NULL;
-  g_free (prop->current_path), prop->current_path = NULL;
-  g_free (prop->virname), prop->virname = NULL;
+  g_free ((*prop)->treeview_path), (*prop)->treeview_path = NULL;
+  if ((*prop)->link_icon != NULL)
+    g_object_unref ((*prop)->link_icon), (*prop)->link_icon = NULL;
+  g_free ((*prop)->user_action), (*prop)->user_action = NULL;
+  g_free ((*prop)->link_value), (*prop)->link_value = NULL;
+  g_free ((*prop)->label), (*prop)->label = NULL;
+  g_free ((*prop)->url), (*prop)->url = NULL;
+  g_free ((*prop)->normalized_url), (*prop)->normalized_url = NULL;
+  g_free ((*prop)->protocol), (*prop)->protocol = NULL;
+  g_free ((*prop)->h_name), (*prop)->h_name = NULL;
+  g_free ((*prop)->port), (*prop)->port = NULL;
+  g_free ((*prop)->path), (*prop)->path = NULL;
+  g_free ((*prop)->domain), (*prop)->domain = NULL;
+  g_free ((*prop)->args), (*prop)->args = NULL;
+  g_free ((*prop)->current_path), (*prop)->current_path = NULL;
+  g_free ((*prop)->virname), (*prop)->virname = NULL;
 
-  if (prop->header != NULL)
-    g_hash_table_destroy (prop->header), prop->header = NULL;
+  if ((*prop)->header != NULL)
+    g_hash_table_destroy ((*prop)->header), (*prop)->header = NULL;
 
-  if (prop->similar_links_parents != NULL)
+  if ((*prop)->similar_links_parents != NULL)
     {
-      g_list_foreach (prop->similar_links_parents, (GFunc) g_free, NULL);
-      g_list_free (prop->similar_links_parents),
-	prop->similar_links_parents = NULL;
+      g_list_foreach ((*prop)->similar_links_parents, (GFunc) g_free, NULL);
+      g_list_free ((*prop)->similar_links_parents),
+	(*prop)->similar_links_parents = NULL;
     }
 
-  if (prop->bad_extensions != NULL)
+  if ((*prop)->bad_extensions != NULL)
     {
-      g_list_foreach (prop->bad_extensions, (GFunc) g_free, NULL);
-      g_list_free (prop->bad_extensions), prop->bad_extensions = NULL;
+      g_list_foreach ((*prop)->bad_extensions, (GFunc) g_free, NULL);
+      g_list_free ((*prop)->bad_extensions), (*prop)->bad_extensions = NULL;
     }
 
-  /* prop->all_links (ptr on prop->tags items) */
-  g_list_free (prop->all_links), prop->all_links = NULL;
-  /* prop->emails (ptr on prop->tags items) */
-  g_list_free (prop->emails), prop->emails = NULL;
-  /* prop->metas ((ptr on prop->tags items) */
-  g_list_free (prop->metas), prop->metas = NULL;
+  /* (*prop)->all_links (ptr on (*prop)->tags items) */
+  g_list_free ((*prop)->all_links), (*prop)->all_links = NULL;
+  /* (*prop)->emails (ptr on (*prop)->tags items) */
+  g_list_free ((*prop)->emails), (*prop)->emails = NULL;
+  /* (*prop)->metas ((ptr on (*prop)->tags items) */
+  g_list_free ((*prop)->metas), (*prop)->metas = NULL;
 
-  if (prop->tags != NULL)
-    uc_lists_html_tags_free (prop->tags), prop->tags = NULL;;
+  if ((*prop)->tags != NULL)
+    uc_lists_html_tags_free (&(*prop)->tags);
 
-  if (prop->childs != NULL)
-    prop->childs = uc_lists_checked_links_free_real (prop->childs);
+  if ((*prop)->childs != NULL)
+    uc_lists_checked_links_free_real (&(*prop)->childs);
 
   if (list != NULL)
-    list = g_list_remove (list, prop);
+    *list = g_list_remove (*list, (gpointer) *prop);
 
-  g_free (prop), prop = NULL;
-
-  return list;
+  g_free (*prop), *prop = NULL;
 }
 
+
 /*
  * free all the allocated objects of the
  * url properties list
  */
-static GList *
-uc_lists_checked_links_free_real (GList * list)
+static void
+uc_lists_checked_links_free_real (GList **list)
 {
   GList *item = NULL;
 
-  if (list == NULL)
-    return list;
 
-  item = g_list_first (list);
+  if (*list == NULL)
+    return;
+
+  item = g_list_first (*list);
   while (item != NULL)
     {
       UCLinkProperties *prop = (UCLinkProperties *) item->data;
-
       item = g_list_next (item);
-
-      list = uc_lists_checked_links_node_free (list, prop);
+      uc_lists_checked_links_node_free (list, &prop);
     }
 
-  g_list_free (list), list = NULL;
-
-  return list;
+  g_list_free (*list), *list = NULL;
 }
 
+
 /*
  * TRUE if the given url already in
  * the checked urls list
@@ -351,7 +343,7 @@ uc_lists_already_checked_links_remove (const gchar * url)
       g_free (node->url), node->url = NULL;
 
       uc_lists_already_checked_links_list =
-	g_list_remove (uc_lists_already_checked_links_list, node);
+	g_list_remove (uc_lists_already_checked_links_list, (gpointer) node);
 
       g_free (node), node = NULL;
     }
@@ -419,7 +411,7 @@ uc_lists_similar_links_remove_item (GList * list, const gchar * url)
 	  if (!g_ascii_strcasecmp (sl, url))
 	    {
 	      prop->similar_links_parents =
-		g_list_remove (prop->similar_links_parents, sl);
+		g_list_remove (prop->similar_links_parents, (gpointer) sl);
 
 	      g_free (sl), sl = NULL;
 	    }
@@ -453,7 +445,7 @@ uc_lists_similar_links_remove_items (UCLinkProperties * prop)
 	  if (!g_ascii_strcasecmp (tmp, lp->url))
 	    {
 	      lp->similar_links_parents =
-		g_list_remove (lp->similar_links_parents, tmp);
+		g_list_remove (lp->similar_links_parents, (gpointer) tmp);
 
 	      g_free (tmp), tmp = NULL;
 	    }
@@ -465,29 +457,17 @@ uc_lists_similar_links_remove_items (UCLinkProperties * prop)
  * Append a url in the similar links list
  */
 void
-uc_lists_similar_links_append (gpointer prop_p, gpointer url_p)
+uc_lists_similar_links_append (gpointer prop_p, const gchar *url)
 {
   UCLinkProperties *prop = (UCLinkProperties *) prop_p;
 
+
   g_return_if_fail (prop != NULL);
 
   prop->similar_links_parents =
-    g_list_append (prop->similar_links_parents, g_strdup (url_p));
+    g_list_append (prop->similar_links_parents, g_strdup (url));
 }
 
-/*
- * Append a url in the bad extensions list
- */
-void
-uc_lists_bad_extensions_append (gpointer prop_p, gpointer url_p)
-{
-  UCLinkProperties *prop = (UCLinkProperties *) prop_p;
-
-  g_return_if_fail (prop != NULL);
-
-  prop->bad_extensions =
-    g_list_append (prop->bad_extensions, g_strdup (url_p));
-}
 
 /*
  * free all allocated url strings
@@ -497,6 +477,7 @@ uc_lists_already_checked_free (void)
 {
   GList *item = NULL;
 
+
   if (uc_lists_already_checked_links_list == NULL)
     return;
 
@@ -504,9 +485,7 @@ uc_lists_already_checked_free (void)
   while (item != NULL)
     {
       UCAlreadyChecked *ac = (UCAlreadyChecked *) item->data;
-
       item = g_list_next (item);
-
       g_free (ac->url), ac->url = NULL;
       g_free (ac), ac = NULL;
     }
@@ -515,6 +494,7 @@ uc_lists_already_checked_free (void)
     uc_lists_already_checked_links_list = NULL;
 }
 
+
 /*
  * allocate and fill a uc_lists_already_checked_links_list node
  */
@@ -551,6 +531,73 @@ uc_lists_checked_links_set (GList * list)
   uc_lists_checked_links_list = list;
 }
 
+void
+uc_lists_checked_links_dump (GList *list, FILE **fd,
+                             const UCExportFormat ftype)
+{
+  GList *item = NULL;
+  UCLinkProperties *lp = NULL;
+  UCHTMLTag *lps = NULL;
+  gboolean labels = uc_project_get_export_labels ();
+  gboolean numbers = uc_project_get_export_numbering ();
+
+
+  g_assert (list != NULL);
+
+  item = g_list_first (list);
+  while (item != NULL)
+  {
+    if (ftype != UC_EXPORT_LIST_SIMPLE)
+    {
+      lp = (UCLinkProperties *) item->data;
+  
+      /* Treepath dump */
+      if (numbers)
+      {
+        fwrite (lp->treeview_path, sizeof (gchar),
+                strlen (lp->treeview_path), *fd);
+        fwrite ("\t", 1, 1, *fd);
+      }
+  
+      /* URL dump */
+      fwrite (lp->normalized_url, sizeof (gchar),
+              strlen (lp->normalized_url), *fd);
+
+      /* Link's label dump */
+      if (labels)
+      {
+        fwrite ("\t", 1, 1, *fd);
+        fwrite (lp->label, sizeof (gchar), strlen (lp->label), *fd);
+      }
+
+      /* EOL */
+      fwrite ("\n", 1, 1, *fd);
+  
+      if (lp->childs != NULL)
+        uc_lists_checked_links_dump (lp->childs, fd, ftype);
+    }
+    else
+    {
+      lps = (UCHTMLTag *) item->data;
+
+      /* URL dump */
+      fwrite (lps->value, sizeof (gchar), strlen (lps->value), *fd);
+
+      /* Link's label dump */
+      if (labels)
+      {
+        fwrite ("\t", 1, 1, *fd);
+        fwrite (lps->label, sizeof (gchar), strlen (lps->label), *fd);
+      }
+
+      /* EOL */
+      fwrite ("\n", 1, 1, *fd);
+    }
+
+    item = g_list_next (item);
+  }
+}
+
 /*
  * return the already checked urls list
  */
