@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "project.h"
 #include "application.h"
 #include "check.h"
 #include "html_parser.h"
@@ -29,6 +28,9 @@
 #include "bookmarks.h"
 #include "cookies.h"
 
+#include "project.h"
+
+
 struct
 {
   guint32 id;
@@ -43,6 +45,7 @@ struct
   gboolean save_bookmarks;
   gchar *url;
   gchar *bookmarks_file;
+  UCBookmarksType bookmarks_type;
   gchar *report_export_path;
   gchar *current_host;
   gchar *current_port;
@@ -86,19 +89,17 @@ struct
   gboolean export_numbering;
   gboolean debug_mode;
   gboolean dump_properties;
+  gchar *reject_images;
+  gchar *reject_documents;
+  gchar *html_extensions;
+  gchar *css_extensions;
+  gchar *virii_extensions;
+  gchar *bad_extensions;
   gchar **reject_directories;
   gchar **reject_domains;
-  gchar **reject_documents;
-  gchar **virii_extensions;
-  gchar **html_extensions;
-  gchar **css_extensions;
-  gchar **reject_images;
-  gchar **bad_extensions;
   gboolean auth_save;
   gboolean no_urls_args;
   gchar *auth_line;
-  gchar *auth_user;
-  gchar *auth_password;
   GHashTable *cookies;
 }
 static project;
@@ -138,7 +139,7 @@ static void uc_project_serialize_tags (xmlTextWriterPtr writer,
 				       UCHTMLTag * main_tag, GList * tags);
 static guint uc_project_get_new_project_id (void);
 static void uc_project_projects_list_free (void);
-static void uc_project_projects_list_node_free (UCProjectProjects * pl);
+static void uc_project_projects_list_node_free (UCProjectProjects **pl);
 static UCProjectProjects *uc_project_projects_list_lookup_by_uid (const gint
 								  id);
 static GList *uc_project_xml_get_tags (const gchar * docname);
@@ -171,7 +172,6 @@ static void uc_project_xml_report_tag (xmlDocPtr doc, xmlNodePtr cur);
 static void uc_project_projects_list_delete (const gint32 id);
 static void uc_project_general_settings_create_default (void);
 static void uc_project_proxy_init (void);
-static void uc_project_browser_init (void);
 static void uc_project_already_checked_links_append_all (GList * list);
 static void
 uc_project_xml_write_attribute (xmlTextWriterPtr writer,
@@ -289,31 +289,34 @@ uc_project_delete (const gint32 id)
     uc_application_init (NULL, NULL, NULL, FALSE);
 }
 
+
 static void
 uc_project_already_checked_links_append_all (GList * list)
 {
   GList *item = NULL;
-  gchar *normalized_url = NULL;
+
 
   item = g_list_first (list);
   while (item != NULL)
-    {
-      UCLinkProperties *prop = (UCLinkProperties *) item->data;
+  {
+    gchar *normalized_url = NULL;
+    UCLinkProperties *prop = (UCLinkProperties *) item->data;
 
-      normalized_url = uc_url_normalize (uc_project_get_current_host (),
-					 (prop->parent !=
-					  NULL) ? prop->parent->path : "",
-					 prop->url);
-      prop->normalized_url = normalized_url;
-      uc_lists_already_checked_links_append (normalized_url, prop);
 
-      if (prop->childs != NULL)
-	uc_project_already_checked_links_append_all (prop->childs);
+    normalized_url = uc_url_normalize (uc_project_get_current_host (),
+                                       (prop->parent != NULL) ?
+                                         prop->parent->path : "", prop->url);
+    prop->normalized_url = normalized_url;
+    uc_lists_already_checked_links_append (normalized_url, prop);
 
-      item = g_list_next (item);
-    }
+    if (prop->childs != NULL)
+      uc_project_already_checked_links_append_all (prop->childs);
+
+    item = g_list_next (item);
+  }
 }
 
+
 /**
  * uc_project_open:
  * @id: id of the project to open.
@@ -333,16 +336,13 @@ uc_project_open (const gint32 id)
   gchar *path = NULL;
   gchar *path2 = NULL;
   GtkWidget *widget = NULL;
+  GtkTreeIter iter;
 
   uc_project_new ();
 
-  //uc_check_reset ();
-
   WSENS ("menu_bar", FALSE);
   WSENS ("tool_bar", FALSE);
 
-  //uc_project_free ();
-
   uc_application_draw_main_frames ();
   UC_UPDATE_UI;
 
@@ -375,8 +375,8 @@ uc_project_open (const gint32 id)
   g_free (path2), path2 = NULL;
   g_free (path), path = NULL;
 
-  /* rebuild already checked global list and normalized_url fiel
-   * (this one for each page) */
+  // rebuild already checked global list and normalized_url field
+  // (this one for each page)
   uc_project_already_checked_links_append_all (uc_lists_checked_links_get ());
 
   /* build tree view and display it */
@@ -390,7 +390,9 @@ uc_project_open (const gint32 id)
 				  G_TYPE_STRING, G_TYPE_STRING,
 				  G_TYPE_STRING);
 
-  uc_check_display_list (uc_lists_checked_links_get (), NULL);
+  UC_DISPLAY_SET_ALL;
+  uc_check_display_list (uc_lists_checked_links_get (), NULL, iter);
+
   gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (treestore));
 
   uc_application_build_url_treeview ();
@@ -400,17 +402,20 @@ uc_project_open (const gint32 id)
   /* FIXME
    * for the moment we are not able to manage projects
    * for bookmarks files... */
-  WSENS ("mwm_project_properties",
-	 (uc_project_get_bookmarks_file () == NULL));
-  WSENS ("mwm_display_security_alerts",
-	 (uc_project_get_bookmarks_file () == NULL));
+  WSENS ("mwm_project_properties", !uc_project_get_check_is_bookmarks ());
   WSENS ("mwm_refresh_all", TRUE);
-  WSENS ("mwm_refresh_main_page", TRUE);
+  WSENS ("mwm_refresh_main_page", !uc_project_get_check_is_bookmarks ());
   WSENS ("mwm_delete_project", TRUE);
   WSENS ("mwm_find", TRUE);
   WSENS ("mwm_display", TRUE);
   WSENS ("mwm_report_export", TRUE);
-  WSENS ("mwm_display_security_alerts", TRUE);
+#ifndef ENABLE_GNUTLS
+  if (strstr (p->location, UC_PROTOCOL_HTTPS))
+  WSENS ("mwm_refresh_main_page", FALSE);
+#endif
+#ifdef ENABLE_CLAMAV
+  WSENS ("mwm_display_security_alerts", !uc_project_get_speed_check ());
+#endif
 
   uc_report_display_update ();
 
@@ -477,8 +482,12 @@ uc_project_treeview_get_selected_row_id (void)
 static void
 uc_project_projects_list_delete (const gint32 id)
 {
-  uc_project_projects_list_node_free (uc_project_projects_list_lookup_by_uid
-				      (id));
+  UCProjectProjects *pl = NULL;
+
+  
+  pl = uc_project_projects_list_lookup_by_uid (id);
+  if (pl != NULL)
+    uc_project_projects_list_node_free (&pl);
 }
 
 /**
@@ -542,10 +551,10 @@ uc_project_display_informations (GtkTreeView * treeview)
 
   pl = uc_project_projects_list_lookup_by_uid (id);
 
-  (size_t) strftime (creation_date, UC_BUFFER_DATE_LEN, "%Y-%m-%d",
-		     localtime (&pl->create));
-  (size_t) strftime (update_date, UC_BUFFER_DATE_LEN, "%Y-%m-%d",
-		     localtime (&pl->update));
+  strftime (creation_date, UC_BUFFER_DATE_LEN, "%Y-%m-%d",
+            localtime (&pl->create));
+  strftime (update_date, UC_BUFFER_DATE_LEN, "%Y-%m-%d",
+            localtime (&pl->update));
 
   widget = WGET ("opd_dates");
   tmp = g_strdup_printf (_("<i>Created on</i> %s\n"
@@ -614,8 +623,11 @@ uc_project_save_properties (void)
 
   gtk_widget_hide (WGET ("project_information_dialog"));
 
-  uc_project_set_save (TRUE);
-  WSENS ("mwm_delete_project", TRUE);
+  if (!uc_project_get_check_is_bookmarks ())
+  {
+    uc_project_set_save (TRUE);
+    WSENS ("mwm_delete_project", TRUE);
+  }
 
   return TRUE;
 }
@@ -762,6 +774,7 @@ static void
 uc_project_serialize_filters (xmlTextWriterPtr writer)
 {
   gchar **tmp = NULL;
+  gchar *tmp1 = NULL;
   guint size = 0;
   guint i = 0;
 
@@ -780,40 +793,52 @@ uc_project_serialize_filters (xmlTextWriterPtr writer)
   xmlTextWriterEndElement (writer);
 
   xmlTextWriterStartElement (writer, BAD_CAST "documents");
-  uc_project_get_reject_documents (&tmp, &size);
-  for (i = 0; i < size; i++)
+  tmp1 = uc_project_get_reject_documents ();
+  tmp = g_strsplit (tmp1, ",", -1);
+  for (i = 0; i < uc_utils_vector_length (tmp); i++)
     uc_project_xml_write_element (writer, "item", tmp[i], 0);
   xmlTextWriterEndElement (writer);
+  g_strfreev (tmp), tmp = NULL;
 
   xmlTextWriterStartElement (writer, BAD_CAST "images");
-  uc_project_get_reject_images (&tmp, &size);
-  for (i = 0; i < size; i++)
+  tmp1 = uc_project_get_reject_images ();
+  tmp = g_strsplit (tmp1, ",", -1);
+  for (i = 0; i < uc_utils_vector_length (tmp); i++)
     uc_project_xml_write_element (writer, "item", tmp[i], 0);
   xmlTextWriterEndElement (writer);
+  g_strfreev (tmp), tmp = NULL;
 
   xmlTextWriterStartElement (writer, BAD_CAST "virii_extensions");
-  uc_project_get_security_virii_extensions (&tmp, &size);
-  for (i = 0; i < size; i++)
+  tmp1 = uc_project_get_security_virii_extensions ();
+  tmp = g_strsplit (tmp1, ",", -1);
+  for (i = 0; i < uc_utils_vector_length (tmp); i++)
     uc_project_xml_write_element (writer, "item", tmp[i], 0);
   xmlTextWriterEndElement (writer);
+  g_strfreev (tmp), tmp = NULL;
 
   xmlTextWriterStartElement (writer, BAD_CAST "html_extensions");
-  uc_project_get_w3c_html_extensions (&tmp, &size);
-  for (i = 0; i < size; i++)
+  tmp1 = uc_project_get_w3c_html_extensions ();
+  tmp = g_strsplit (tmp1, ",", -1);
+  for (i = 0; i < uc_utils_vector_length (tmp); i++)
     uc_project_xml_write_element (writer, "item", tmp[i], 0);
   xmlTextWriterEndElement (writer);
+  g_strfreev (tmp), tmp = NULL;
 
   xmlTextWriterStartElement (writer, BAD_CAST "css_extensions");
-  uc_project_get_w3c_css_extensions (&tmp, &size);
-  for (i = 0; i < size; i++)
+  tmp1 = uc_project_get_w3c_css_extensions ();
+  tmp = g_strsplit (tmp1, ",", -1);
+  for (i = 0; i < uc_utils_vector_length (tmp); i++)
     uc_project_xml_write_element (writer, "item", tmp[i], 0);
   xmlTextWriterEndElement (writer);
+  g_strfreev (tmp), tmp = NULL;
 
   xmlTextWriterStartElement (writer, BAD_CAST "bad_extensions");
-  uc_project_get_security_bad_extensions (&tmp, &size);
-  for (i = 0; i < size; i++)
+  tmp1 = uc_project_get_security_bad_extensions ();
+  tmp = g_strsplit (tmp1, ",", -1);
+  for (i = 0; i < uc_utils_vector_length (tmp); i++)
     uc_project_xml_write_element (writer, "item", tmp[i], 0);
   xmlTextWriterEndElement (writer);
+  g_strfreev (tmp), tmp = NULL;
 
   xmlTextWriterEndElement (writer);
 }
@@ -967,6 +992,7 @@ uc_project_save_index (void)
   g_free (path), path = NULL;
 }
 
+
 /**
  * uc_project_general_settings_create_default:
  *
@@ -980,6 +1006,7 @@ uc_project_general_settings_create_default (void)
   xmlDocPtr doc = NULL;
   gchar *path = NULL;
 
+
   path = g_strdup_printf ("%s/settings.xml", uc_project_get_working_path ());
 
   writer = xmlNewTextWriterDoc (&doc, 0);
@@ -1007,7 +1034,8 @@ uc_project_general_settings_create_default (void)
   uc_project_xml_write_attribute (writer, "proto_ftp", NULL, 1, TRUE);
   uc_project_xml_write_attribute (writer, "passive_ftp", NULL, 1, TRUE);
   uc_project_xml_write_attribute (writer, "check_chroot", NULL, 0, TRUE);
-  uc_project_xml_write_attribute (writer, "limit_local", NULL, 0, TRUE);
+  uc_project_xml_write_attribute (writer, "limit_local", NULL, 1, TRUE);
+  uc_project_xml_write_attribute (writer, "no_urls_args", NULL, 1, TRUE);
   uc_project_xml_write_attribute (writer, "use_proxy", NULL,
 				  uc_project_get_use_proxy (), TRUE);
   uc_project_xml_write_attribute (writer, "proxy_host",
@@ -1059,6 +1087,7 @@ uc_project_general_settings_create_default (void)
   g_free (path), path = NULL;
 }
 
+
 /**
  * uc_project_xml_write_attribute:
  * @writer: #xmlTextWriterPtr from libxml2 API.
@@ -1168,6 +1197,8 @@ uc_project_general_settings_save (void)
 				  uc_project_get_check_chroot (), TRUE);
   uc_project_xml_write_attribute (writer, "limit_local", NULL,
 				  uc_project_get_limit_local (), TRUE);
+  uc_project_xml_write_attribute (writer, "no_urls_args", NULL,
+				  uc_project_get_no_urls_args (), TRUE);
   uc_project_xml_write_attribute (writer, "use_proxy", NULL,
 				  uc_project_get_use_proxy (), TRUE);
   uc_project_xml_write_attribute (writer, "proxy_host",
@@ -1331,6 +1362,8 @@ uc_project_save_all (void)
  *
  * Save the entire current project (settings, links etc.) in XML format
  * on the disk.
+ *
+ * Returns: %TRUE if project must be saved.
  */
 gint
 uc_project_save (void)
@@ -1342,15 +1375,15 @@ uc_project_save (void)
   gint res = 0;
   UCProjectProjects *node = NULL;
 
-  if (uc_project_get_bookmarks_file () != NULL)
+  if (uc_project_get_check_is_bookmarks ())
     return uc_bookmarks_save_changes ();
 
-  res = uc_application_dialog_yes_no_show (_("Save the current project?"),
+  res = uc_application_dialog_yes_no_show (_("Save the current project ?"),
 					   GTK_MESSAGE_QUESTION);
   if (res != GTK_RESPONSE_YES)
     return res;
 
-  if (!strlen (uc_project_get_title ()) &&
+  if (strlen (uc_project_get_title ()) == 0 &&
       !uc_application_project_information_dialog_show ())
     return GTK_RESPONSE_CANCEL;
 
@@ -1367,7 +1400,7 @@ uc_project_save (void)
       node->type = uc_project_get_type ();
       node->create = t;
       node->update = t;
-      node->location = (uc_project_get_bookmarks_file () != NULL) ?
+      node->location = (uc_project_get_check_is_bookmarks ()) ?
 	g_strdup (uc_project_get_bookmarks_file ()) :
 	g_strdup (uc_project_get_url ());
       node->title = g_strdup (uc_project_get_title ());
@@ -1435,17 +1468,19 @@ uc_project_save (void)
  * Free a projects' list node.
  */
 static void
-uc_project_projects_list_node_free (UCProjectProjects * pl)
+uc_project_projects_list_node_free (UCProjectProjects ** pl)
 {
-  g_free (pl->location), pl->location = NULL;
-  g_free (pl->title), pl->title = NULL;
-  g_free (pl->description), pl->description = NULL;
+  g_free ((*pl)->location), (*pl)->location = NULL;
+  g_free ((*pl)->title), (*pl)->title = NULL;
+  g_free ((*pl)->description), (*pl)->description = NULL;
 
-  uc_project_projects_list = g_list_remove (uc_project_projects_list, pl);
+  uc_project_projects_list =
+    g_list_remove (uc_project_projects_list, (gpointer) *pl);
 
-  g_free (pl), pl = NULL;
+  g_free (*pl), *pl = NULL;
 }
 
+
 /**
  * uc_project_projects_list_free:
  *
@@ -1456,22 +1491,25 @@ uc_project_projects_list_free (void)
 {
   GList *item = NULL;
 
+
   if (uc_project_projects_list == NULL)
     return;
 
   item = g_list_first (uc_project_get_projects_list ());
   while (item != NULL)
-    {
-      UCProjectProjects *pl = (UCProjectProjects *) item->data;
+  {
+    UCProjectProjects *pl = (UCProjectProjects *) item->data;
 
-      item = g_list_next (item);
 
-      uc_project_projects_list_node_free (pl);
-    }
+    item = g_list_next (item);
+
+    uc_project_projects_list_node_free (&pl);
+  }
 
   g_list_free (uc_project_projects_list), uc_project_projects_list = NULL;
 }
 
+
 /**
  * uc_project_projects_list_load:
  *
@@ -1558,17 +1596,15 @@ uc_project_new (void)
 void
 uc_project_free (void)
 {
-  uc_application_set_status_bar (0, _("Cleaning..."));
-  UC_UPDATE_UI;
-
   g_strfreev (project.reject_directories), project.reject_directories = NULL;
   g_strfreev (project.reject_domains), project.reject_domains = NULL;
-  g_strfreev (project.virii_extensions), project.virii_extensions = NULL;
-  g_strfreev (project.html_extensions), project.html_extensions = NULL;
-  g_strfreev (project.css_extensions), project.css_extensions = NULL;
-  g_strfreev (project.reject_documents), project.reject_documents = NULL;
-  g_strfreev (project.reject_images), project.reject_images = NULL;
-  g_strfreev (project.bad_extensions), project.bad_extensions = NULL;
+
+  g_free (project.reject_documents), project.reject_documents = NULL;
+  g_free (project.reject_images), project.reject_images = NULL;
+  g_free (project.html_extensions), project.html_extensions = NULL;
+  g_free (project.css_extensions), project.css_extensions = NULL;
+  g_free (project.virii_extensions), project.virii_extensions = NULL;
+  g_free (project.bad_extensions), project.bad_extensions = NULL;
 
   g_free (project.browser_path), project.browser_path = NULL;
   g_free (project.description), project.description = NULL;
@@ -1584,8 +1620,6 @@ uc_project_free (void)
 
   g_free (project.proxy_host), project.proxy_host = NULL;
   g_free (project.auth_line), project.auth_line = NULL;
-  g_free (project.auth_user), project.auth_user = NULL;
-  g_free (project.auth_password), project.auth_password = NULL;
   g_free (project.chroot_path), project.chroot_path = NULL;
 
   uc_project_projects_list_free ();
@@ -1593,11 +1627,7 @@ uc_project_free (void)
   project.id = 0;
   project.local_charset = NULL;
 
-  uc_cookies_free ();
-
   uc_project_set_save_bookmarks (FALSE);
-
-  uc_application_set_status_bar (0, "");
 }
 
 /**
@@ -1605,14 +1635,20 @@ uc_project_free (void)
  *
  * Check for the browser path and return it.
  * 
- * See: uc_project_browser_init ()
- * 
  * Returns: The browser path (no new memory allocation).
  */
 gchar *
 uc_project_get_browser_path (void)
 {
-  uc_project_browser_init ();
+  gchar *browser = NULL;
+
+
+  if (project.browser_path == NULL)
+  {
+    browser = uc_utils_get_gnome_browser_conf ();
+    uc_project_set_browser_path (browser);
+    g_free (browser), browser = NULL;
+  }
 
   return project.browser_path;
 }
@@ -1724,11 +1760,8 @@ uc_project_get_reject_domains (gchar *** items, guint * item_size)
 
 /**
  * uc_project_get_security_virii_extensions:
- * @items: the vector to fill.
- * @item_size: the size to fill.
  *
- * Fill the @items vector with the virii extensions to scan during
- * the check.
+ * Return a string with comma separated extensions to scan during the check.
  *
  * There is no memory allocation.
  *
@@ -1739,30 +1772,21 @@ uc_project_get_reject_domains (gchar *** items, guint * item_size)
  *      uc_project_get_security_bad_extensions (),
  *      uc_project_get_w3c_html_extensions (),
  *      uc_project_get_w3c_css_extensions ()
+ *
+ * Returns: A string with comma separated extensions.
  */
-void
-uc_project_get_security_virii_extensions (gchar *** items, guint * item_size)
+gchar *
+uc_project_get_security_virii_extensions (void)
 {
-  if (project.virii_extensions != NULL)
-    {
-      *items = project.virii_extensions;
-      *item_size = uc_utils_vector_length (project.virii_extensions);
-    }
-  else
-    {
-      uc_project_set_security_virii_extensions
-	(UC_DEFAULT_SECURITY_VIRII_EXTENSIONS);
-      uc_project_get_security_virii_extensions (items, item_size);
-    }
+  return (project.virii_extensions != NULL)?
+    project.virii_extensions:UC_DEFAULT_SECURITY_VIRII_EXTENSIONS;
 }
 
 /**
  * uc_project_get_w3c_html_extensions:
- * @items: the vector to fill.
- * @item_size: the size to fill.
  *
- * Fill the @items vector with the html extensions to w3c validate during
- * the check.
+ * Return a string with comma separated extensions of HTML documents to
+ * validate during the scan.
  *
  * There is no memory allocation.
  *
@@ -1773,29 +1797,22 @@ uc_project_get_security_virii_extensions (gchar *** items, guint * item_size)
  *      uc_project_get_security_bad_extensions (),
  *      uc_project_get_security_virii_extensions (),
  *      uc_project_get_w3c_css_extensions ()
+ *
+ * Returns: A string with comma separated extensions.
  */
-void
-uc_project_get_w3c_html_extensions (gchar *** items, guint * item_size)
+gchar *
+uc_project_get_w3c_html_extensions (void)
 {
-  if (project.html_extensions != NULL)
-    {
-      *items = project.html_extensions;
-      *item_size = uc_utils_vector_length (project.html_extensions);
-    }
-  else
-    {
-      uc_project_set_w3c_html_extensions (UC_DEFAULT_W3C_HTML_EXTENSIONS);
-      uc_project_get_w3c_html_extensions (items, item_size);
-    }
+  return (project.html_extensions != NULL)?
+    project.html_extensions:UC_DEFAULT_W3C_HTML_EXTENSIONS;
 }
 
+
 /**
  * uc_project_get_w3c_css_extensions:
- * @items: the vector to fill.
- * @item_size: the size to fill.
  *
- * Fill the @items vector with the css extensions to w3c validate during
- * the check.
+ * Return a string with comma separated extensions of CSS files to validate
+ * during the scan.
  *
  * There is no memory allocation.
  *
@@ -1806,31 +1823,20 @@ uc_project_get_w3c_html_extensions (gchar *** items, guint * item_size)
  *      uc_project_get_security_bad_extensions (),
  *      uc_project_get_security_virii_extensions (),
  *      uc_project_get_w3c_html_extensions ()
+ *
+ * Returns: A string with comma separated extensions.
  */
-void
-uc_project_get_w3c_css_extensions (gchar *** items, guint * item_size)
+gchar *
+uc_project_get_w3c_css_extensions (void)
 {
-  if (project.css_extensions != NULL)
-    {
-      *items = project.css_extensions;
-      *item_size = uc_utils_vector_length (project.css_extensions);
-    }
-  else
-    {
-      uc_project_set_w3c_css_extensions (UC_DEFAULT_W3C_CSS_EXTENSIONS);
-      uc_project_get_w3c_css_extensions (items, item_size);
-    }
+  return (project.css_extensions != NULL)?
+    project.css_extensions:UC_DEFAULT_W3C_CSS_EXTENSIONS;
 }
 
 /**
  * uc_project_get_reject_documents:
- * @items: the vector to fill.
- * @item_size: the size to fill.
  *
- * Fill the @items vector with the documents extensions to avoid during
- * the check (those documents will not be checked).
- *
- * There is no memory allocation.
+ * Return a string with comma separated extensions to ignore during the scan.
  *
  * See: uc_project_get_reject_directories (),
  *      uc_project_get_reject_domains (),
@@ -1839,29 +1845,22 @@ uc_project_get_w3c_css_extensions (gchar *** items, guint * item_size)
  *      uc_project_get_security_virii_extensions (),
  *      uc_project_get_w3c_html_extensions (),
  *      uc_project_get_w3c_css_extensions ()
+ *
+ * Returns: A string with comma separated extensions.
  */
-void
-uc_project_get_reject_documents (gchar *** items, guint * item_size)
+gchar *
+uc_project_get_reject_documents (void)
 {
-  if (project.reject_documents != NULL)
-    {
-      *items = project.reject_documents;
-      *item_size = uc_utils_vector_length (project.reject_documents);
-    }
-  else
-    {
-      *items = NULL;
-      *item_size = 0;
-    }
+  if (project.reject_documents == NULL)
+    uc_project_set_reject_documents ("");
+
+  return project.reject_documents;
 }
 
 /**
  * uc_project_get_reject_images:
- * @items: the vector to fill.
- * @item_size: the size to fill.
  *
- * Fill the @items vector with the images extensions to avoid during
- * the check (those images will not be checked).
+  Return a string with comma separated extensions to ignore during the scan.
  *
  * There is no memory allocation.
  *
@@ -1872,29 +1871,23 @@ uc_project_get_reject_documents (gchar *** items, guint * item_size)
  *      uc_project_get_security_virii_extensions (),
  *      uc_project_get_w3c_html_extensions (),
  *      uc_project_get_w3c_css_extensions ()
+ *
+ * Returns: A string with comma separated extensions.
  */
-void
-uc_project_get_reject_images (gchar *** items, guint * item_size)
+gchar *
+uc_project_get_reject_images (void)
 {
-  if (project.reject_images != NULL)
-    {
-      *items = project.reject_images;
-      *item_size = uc_utils_vector_length (project.reject_images);
-    }
-  else
-    {
-      *items = NULL;
-      *item_size = 0;
-    }
+  if (project.reject_images == NULL)
+    uc_project_set_reject_images ("");
+
+  return project.reject_images;
 }
 
 /**
  * uc_project_get_security_bad_extensions:
- * @items: the vector to fill.
- * @item_size: the size to fill.
  *
- * Fill the @items vector with the bad extensions to warn for during
- * the check (those extensions will be explicitly searched).
+ * Return a string with comma separated extensions of files to search during
+ * the security scan.
  *
  * There is no memory allocation.
  *
@@ -1905,32 +1898,16 @@ uc_project_get_reject_images (gchar *** items, guint * item_size)
  *      uc_project_get_security_virii_extensions (),
  *      uc_project_get_w3c_html_extensions (),
  *      uc_project_get_w3c_css_extensions ()
+ *
+ * Returns: A string with comma separated extensions.
  */
-void
-uc_project_get_security_bad_extensions (gchar *** items, guint * item_size)
+gchar *
+uc_project_get_security_bad_extensions (void)
 {
-  if (project.bad_extensions != NULL)
-    {
-      *items = project.bad_extensions;
-      *item_size = uc_utils_vector_length (project.bad_extensions);
-    }
-  else
-    {
-      uc_project_set_security_bad_extensions
-	(UC_DEFAULT_SECURITY_BAD_EXTENSIONS);
-      uc_project_get_security_bad_extensions (items, item_size);
-    }
+  return (project.bad_extensions != NULL)?
+    project.bad_extensions:UC_DEFAULT_SECURITY_BAD_EXTENSIONS;
 }
 
-static void
-uc_project_browser_init (void)
-{
-  gchar *browser = NULL;
-
-  browser = uc_utils_get_gnome_browser_conf ();
-  uc_project_set_browser_path (browser);
-  g_free (browser), browser = NULL;
-}
 
 static void
 uc_project_proxy_init (void)
@@ -2012,6 +1989,12 @@ uc_project_get_check_is_main (void)
   return project.check_is_main;
 }
 
+gboolean
+uc_project_get_check_is_bookmarks (void)
+{
+  return (project.bookmarks_type != UC_BOOKMARKS_TYPE_NONE);
+}
+
 gchar *
 uc_project_get_url (void)
 {
@@ -2024,6 +2007,12 @@ uc_project_get_bookmarks_file (void)
   return project.bookmarks_file;
 }
 
+UCBookmarksType
+uc_project_get_bookmarks_type (void)
+{
+  return project.bookmarks_type;
+}
+
 gchar *
 uc_project_get_report_export_path (void)
 {
@@ -2164,7 +2153,7 @@ uc_project_get_passive_ftp (void)
 gboolean
 uc_project_speed_check_get_download_content (void)
 {
-  return (uc_project_get_bookmarks_file () == NULL) ?
+  return (!uc_project_get_check_is_bookmarks ()) ?
     uc_project_speed_check_set_download_content_value : FALSE;
 }
 
@@ -2232,18 +2221,6 @@ uc_project_get_auth_line (void)
   return project.auth_line;
 }
 
-gchar *
-uc_project_get_auth_user (void)
-{
-  return project.auth_user;
-}
-
-gchar *
-uc_project_get_auth_password (void)
-{
-  return project.auth_password;
-}
-
 gchar *
 uc_project_get_chroot_path (void)
 {
@@ -2377,43 +2354,43 @@ uc_project_set_reject_domains (const gchar * items)
 void
 uc_project_set_security_virii_extensions (const gchar * items)
 {
-  g_strfreev (project.virii_extensions), project.virii_extensions = NULL;
-  project.virii_extensions = g_strsplit (items, ",", -1);
+  g_free (project.virii_extensions), project.virii_extensions = NULL;
+  project.virii_extensions = g_strdup (items);
 }
 
 void
 uc_project_set_w3c_html_extensions (const gchar * items)
 {
-  g_strfreev (project.html_extensions), project.html_extensions = NULL;
-  project.html_extensions = g_strsplit (items, ",", -1);
+  g_free (project.html_extensions), project.html_extensions = NULL;
+  project.html_extensions = g_strdup (items);
 }
 
 void
 uc_project_set_w3c_css_extensions (const gchar * items)
 {
-  g_strfreev (project.css_extensions), project.css_extensions = NULL;
-  project.css_extensions = g_strsplit (items, ",", -1);
+  g_free (project.css_extensions), project.css_extensions = NULL;
+  project.css_extensions = g_strdup (items);
 }
 
 void
 uc_project_set_reject_documents (const gchar * items)
 {
-  g_strfreev (project.reject_documents), project.reject_documents = NULL;
-  project.reject_documents = g_strsplit (items, ",", -1);
+  g_free (project.reject_documents), project.reject_documents = NULL;
+  project.reject_documents = g_strdup (items);
 }
 
 void
 uc_project_set_reject_images (const gchar * items)
 {
-  g_strfreev (project.reject_images), project.reject_images = NULL;
-  project.reject_images = g_strsplit (items, ",", -1);
+  g_free (project.reject_images), project.reject_images = NULL;
+  project.reject_images = g_strdup (items);
 }
 
 void
 uc_project_set_security_bad_extensions (const gchar * items)
 {
-  g_strfreev (project.bad_extensions), project.bad_extensions = NULL;
-  project.bad_extensions = g_strsplit (items, ",", -1);
+  g_free (project.bad_extensions), project.bad_extensions = NULL;
+  project.bad_extensions = g_strdup (items);
 }
 
 void
@@ -2442,17 +2419,32 @@ uc_project_set_speed_check (const gboolean value)
   project.speed_check = value;
 }
 
+
 void
 uc_project_set_browser_path (const gchar * value)
 {
-  gchar *browser = NULL;
+  gchar *p = NULL;
 
-  browser = uc_utils_replace (value, "%s", " ");
 
   g_free (project.browser_path), project.browser_path = NULL;
-  project.browser_path = browser;
+  p = project.browser_path = g_strdup (value);
+ 
+  // Remove extra characters added by GNOME after navigator options
+  // if any
+  do
+  {
+    if (*p == '"')
+      *p = ' ';
+    else if (*p == '%' && *(p + 1) == 's')
+    {
+      *p = ' ';
+      *(p + 1) = ' ';
+    }
+  }
+  while (*(++p));
 }
 
+
 void
 uc_project_set_description (const gchar * value)
 {
@@ -2479,21 +2471,9 @@ uc_project_set_save_bookmarks (const gboolean value)
 void
 uc_project_set_save (const gboolean value)
 {
-  /* FIXME
-   * for the moment we are not able to manage projects
-   * for bookmarks file... */
-  if (uc_project_get_bookmarks_file () == NULL)
-    {
-      project.save = value;
-      WSENS ("mwm_save_project", value);
-      WSENS ("mw_bt_save_project", value);
-    }
-  else
-    {
-      project.save = FALSE;
-      WSENS ("mwm_save_project", FALSE);
-      WSENS ("mw_bt_save_project", TRUE);
-    }
+  project.save = value;
+  WSENS ("mwm_save_project", value);
+  WSENS ("mw_bt_save_project", value);
 }
 
 void
@@ -2534,6 +2514,12 @@ uc_project_set_bookmarks_file (const gchar * value)
   project.bookmarks_file = g_strdup (value);
 }
 
+void
+uc_project_set_bookmarks_type (const UCBookmarksType value)
+{
+  project.bookmarks_type = value;
+}
+
 void
 uc_project_set_report_export_path (const gchar * value)
 {
@@ -2693,20 +2679,6 @@ uc_project_set_auth_line (const gchar * value)
   project.auth_line = g_strdup (value);
 }
 
-void
-uc_project_set_auth_user (const gchar * value)
-{
-  g_free (project.auth_user), project.auth_user = NULL;
-  project.auth_user = g_strdup (value);
-}
-
-void
-uc_project_set_auth_password (const gchar * value)
-{
-  g_free (project.auth_password), project.auth_password = NULL;
-  project.auth_password = g_strdup (value);
-}
-
 void
 uc_project_set_chroot_path (const gchar * value)
 {
@@ -3170,7 +3142,7 @@ uc_project_xml_filters_get_value (xmlDocPtr doc, xmlNodePtr cur)
       if (!xmlStrcmp (cur->name, BAD_CAST "item"))
 	{
 	  value = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
-	  g_string_append_printf (filters, ",%s", value);
+	  g_string_append_printf (filters, ",%s", g_strstrip ((gchar *) value));
 	  xmlFree (value), value = NULL;
 	}
 
@@ -3190,29 +3162,29 @@ uc_project_xml_filters_tag (xmlDocPtr doc, xmlNodePtr cur)
   while (cur != NULL)
     {
       if (!xmlStrcmp (cur->name, BAD_CAST "directories"))
-	uc_project_set_reject_directories (uc_project_xml_filters_get_value
-					   (doc, cur));
+	uc_project_set_reject_directories (
+          uc_project_xml_filters_get_value (doc, cur));
       else if (!xmlStrcmp (cur->name, BAD_CAST "domains"))
-	uc_project_set_reject_domains (uc_project_xml_filters_get_value
-				       (doc, cur));
+	uc_project_set_reject_domains (
+          uc_project_xml_filters_get_value (doc, cur));
       else if (!xmlStrcmp (cur->name, BAD_CAST "documents"))
-	uc_project_set_reject_documents (uc_project_xml_filters_get_value
-					 (doc, cur));
+	uc_project_set_reject_documents (
+          uc_project_xml_filters_get_value (doc, cur));
       else if (!xmlStrcmp (cur->name, BAD_CAST "images"))
-	uc_project_set_reject_images (uc_project_xml_filters_get_value
-				      (doc, cur));
+	uc_project_set_reject_images (
+          uc_project_xml_filters_get_value (doc, cur));
       else if (!xmlStrcmp (cur->name, BAD_CAST "virii_extensions"))
-	uc_project_set_security_virii_extensions
-	  (uc_project_xml_filters_get_value (doc, cur));
+	uc_project_set_security_virii_extensions (
+          uc_project_xml_filters_get_value (doc, cur));
       else if (!xmlStrcmp (cur->name, BAD_CAST "bad_extensions"))
-	uc_project_set_security_bad_extensions
-	  (uc_project_xml_filters_get_value (doc, cur));
+	uc_project_set_security_bad_extensions (
+          uc_project_xml_filters_get_value (doc, cur));
       else if (!xmlStrcmp (cur->name, BAD_CAST "html_extensions"))
-	uc_project_set_w3c_html_extensions (uc_project_xml_filters_get_value
-					    (doc, cur));
+	uc_project_set_w3c_html_extensions (
+          uc_project_xml_filters_get_value (doc, cur));
       else if (!xmlStrcmp (cur->name, BAD_CAST "css_extensions"))
-	uc_project_set_w3c_css_extensions (uc_project_xml_filters_get_value
-					   (doc, cur));
+	uc_project_set_w3c_css_extensions (
+          uc_project_xml_filters_get_value (doc, cur));
 
       cur = cur->next;
     }
@@ -3490,6 +3462,18 @@ uc_project_xml_load_settings (const UCProjectProjects * p)
 	      xmlFree (tmp), tmp = NULL;
 	    }
 
+	  if ((tmp = xmlGetProp (cur, BAD_CAST "no_urls_args")))
+	    {
+	      uc_project_set_no_urls_args (uc_utils_get_yesno ((gchar *) tmp));
+	      xmlFree (tmp), tmp = NULL;
+	    }
+
+	  if ((tmp = xmlGetProp (cur, BAD_CAST "speed_check")))
+	    {
+	      uc_project_set_speed_check (uc_utils_get_yesno ((gchar *) tmp));
+	      xmlFree (tmp), tmp = NULL;
+	    }
+
 	  if ((tmp = xmlGetProp (cur, BAD_CAST "use_proxy")))
 	    {
 	      uc_project_set_use_proxy (uc_utils_get_yesno ((gchar *) tmp));
