@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,10 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "utils.h"
-#include "check.h"
 #include "project.h"
 #include "application.h"
+#include "lists.h"
 
 #include "report.h"
 
@@ -45,7 +44,8 @@ static void report_set_progress_bar (const guint32 report_item,
 				     const gchar * glade_label,
 				     const gchar * report_label,
 				     const gchar * color);
-static gboolean uc_report_export_dialog_show (void);
+static gboolean uc_report_export_dialog_show (const UCExportFormat ftype);
+
 
 /**
  * uc_report_export_dialog_show:
@@ -53,181 +53,213 @@ static gboolean uc_report_export_dialog_show (void);
  * Show the report export dialog box.
  */
 static gboolean
-uc_report_export_dialog_show (void)
+uc_report_export_dialog_show (const UCExportFormat ftype)
 {
   GtkWidget *widget = NULL;
   gint res = 0;
   gboolean ret = FALSE;
   GtkWidget *dialog = NULL;
 
+
   dialog = WGET ("report_export_dialog");
 
   widget = WGET ("report_export_path");
   if (uc_project_get_report_export_path () != NULL)
-    {
-      gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (widget),
-					 uc_project_get_report_export_path
-					 ());
-      gtk_entry_set_text (GTK_ENTRY
-			  (gnome_file_entry_gtk_entry
-			   (GNOME_FILE_ENTRY (widget))),
-			  uc_project_get_report_export_path ());
-    }
+  {
+    gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (widget),
+                                       uc_project_get_report_export_path ());
+    gtk_entry_set_text (
+      GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (widget))),
+      uc_project_get_report_export_path ());
+  }
 
   widget = WGET ("red_export_labels");
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
-				uc_project_get_export_labels ());
-
+                                uc_project_get_export_labels ());
+  
   widget = WGET ("red_export_numbering");
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
-				uc_project_get_export_numbering ());
+                                uc_project_get_export_numbering ());
 
   gtk_widget_show_all (dialog);
 
+  if (ftype == UC_EXPORT_LIST_SIMPLE)
+  {
+    widget = WGET ("red_export_numbering");
+    gtk_widget_hide (widget);
+  }
+
 again:
 
   res = gtk_dialog_run (GTK_DIALOG (dialog));
   if (res == GTK_RESPONSE_OK)
-    {
-      const gchar *path = NULL;
+  {
+    const gchar *path = NULL;
 
-      widget = WGET ("red_export_labels");
-      uc_project_set_export_labels ((gboolean)
-				    gtk_toggle_button_get_active
-				    (GTK_TOGGLE_BUTTON (widget)));
 
-      widget = WGET ("red_export_numbering");
-      uc_project_set_export_numbering ((gboolean)
-				       gtk_toggle_button_get_active
-				       (GTK_TOGGLE_BUTTON (widget)));
+    widget = WGET ("red_export_labels");
+    uc_project_set_export_labels (
+      (gboolean) gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
 
-      widget = WGET ("report_export_path");
-      path =
-	gtk_entry_get_text (GTK_ENTRY
-			    (gnome_file_entry_gtk_entry
-			     (GNOME_FILE_ENTRY (widget))));
+    widget = WGET ("red_export_numbering");
+    uc_project_set_export_numbering (
+      (gboolean) gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
 
-      if (!g_file_test (path, G_FILE_TEST_IS_DIR))
-	goto again;
+    widget = WGET ("report_export_path");
+    path = gtk_entry_get_text (
+      GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (widget))));
 
-      uc_project_set_report_export_path (path);
+    if (!g_file_test (path, G_FILE_TEST_IS_DIR))
+      goto again;
 
-      ret = TRUE;
-    }
+    uc_project_set_report_export_path (path);
+
+    // Add input to export path history
+    gnome_entry_append_history (GNOME_ENTRY (gnome_file_entry_gnome_entry (
+                                               GNOME_FILE_ENTRY (widget))),
+                                TRUE, path);
+
+    ret = TRUE;
+  }
 
   gtk_widget_hide (dialog);
 
   return ret;
 }
 
+
 /**
  * uc_report_export:
+ * @format: export format.
+ * @list: list of URL to export.
+ * @ftype: format type (%UC_EXPORT_NORMAL or %UC_EXPORT_SIMPLE).
  *
  * Export check report in the given format.
  */
 void
-uc_report_export (const UCExportFormat format)
+uc_report_export (const UCExportFormat format, GList *list,
+                  const UCExportFormat ftype)
 {
-  gchar *ext = NULL;
   gchar *message = NULL;
   GError *err = NULL;
-  gchar *command_line = NULL;
   gchar *path = NULL;
   gint32 ret = 0;
   time_t t;
   struct tm *td;
-  gchar creation_date[UC_BUFFER_DATETIME_LEN + 1] = { 0 };
 
-  if (!uc_report_export_dialog_show ())
+
+  if (!uc_report_export_dialog_show (ftype))
     return;
 
-  switch (format)
-    {
-      /* HTML */
-    case UC_EXPORT_HTML:
-      ext = "html";
-      break;
-    default:
-      g_assert_not_reached ();
-    }
-
   time (&t);
   td = localtime (&t);
-  (size_t) strftime (creation_date, UC_BUFFER_DATETIME_LEN,
-		     "%Y-%m-%d %H:%M:%S", td);
-  path = g_strdup_printf ("%s/projects/%u/project.xml",
-			  uc_project_get_working_path (),
-			  uc_project_get_id ());
 
-  command_line = g_strdup_printf (" xsltproc "
-				  " --stringparam date \"%s\" "
-				  " --param withlabel %u "
-				  " --param withnum %u "
-				  " -o %s/uc_report_%u_%02d%02d%02d.%s "
-				  " " UC_STYLESHEETS_DIR "/uc_%s.xsl %s",
-				  creation_date,
-				  (gint) uc_project_get_export_labels (),
-				  (gint) uc_project_get_export_numbering (),
-				  uc_project_get_report_export_path (),
-				  uc_project_get_id (),
-				  td->tm_hour,
-				  td->tm_min, td->tm_sec, ext, ext, path);
+  // HTML
+  if (format == UC_EXPORT_HTML)
+  {
+    gchar creation_date[UC_BUFFER_DATETIME_LEN + 1] = { 0 };
+    gchar *command_line = NULL;
+
+
+    strftime (creation_date, UC_BUFFER_DATETIME_LEN, "%Y-%m-%d %H:%M:%S", td);
+    path = g_strdup_printf ("%s/projects/%u/project.xml",
+                            uc_project_get_working_path (),
+                            uc_project_get_id ());
+
+    command_line = g_strdup_printf (" xsltproc "
+				    " --stringparam date \"%s\" "
+				    " --param withlabel %u "
+				    " --param withnum %u "
+				    " -o %s/uc_report_%u_%02d%02d%02d.html "
+				    " " UC_STYLESHEETS_DIR "/uc_html.xsl %s",
+				    creation_date,
+				    (gint) uc_project_get_export_labels (),
+				    (gint) uc_project_get_export_numbering (),
+				    uc_project_get_report_export_path (),
+				    uc_project_get_id (),
+				    td->tm_hour, td->tm_min, td->tm_sec,
+                                    path);
+
+    ret = g_spawn_command_line_async (command_line, &err);
+    g_free (command_line), command_line = NULL;
+  }
+  // CSV
+  else if (format == UC_EXPORT_CSV)
+  {
+    FILE *fd = NULL;
+
+
+    path = g_strdup_printf (
+      "%s/uc_report_%u_%02d%02d%02d.csv",
+      uc_project_get_report_export_path (),
+      uc_project_get_id (),
+      td->tm_hour, td->tm_min, td->tm_sec);
+
+    fd = fopen (path, "w");
+    uc_lists_checked_links_dump (list, &fd, ftype);
+    fclose (fd);
+
+    ret = 1;
+  }
 
   g_free (path), path = NULL;
 
-  ret = g_spawn_command_line_async (command_line, &err);
-  g_free (command_line), command_line = NULL;
-
   if (ret == 0)
-    {
-      message = g_strdup_printf ("%s\n\n%s",
-				 _
-				 ("Cannot execute xsltproc; the following error occured:"),
-				 err->message);
+  {
+    message = g_strdup_printf (
+      "%s\n\n%s",
+      _("Cannot execute xsltproc; the following error occured:"), err->message);
 
-      uc_application_dialog_show (message, GTK_MESSAGE_ERROR);
+    uc_application_dialog_show (message, GTK_MESSAGE_ERROR);
 
-      g_clear_error (&err);
-      g_free (message), message = NULL;
-    }
+    g_clear_error (&err);
+    g_free (message), message = NULL;
+  }
 }
 
+
 guint32
 uc_report_get_malformedlinks (void)
 {
   return projectReport.malformedLinks;
 }
 
+
 guint32
 uc_report_get_badlinks (void)
 {
   return projectReport.badLinks;
 }
 
+
 guint32
 uc_report_get_ignoredlinks (void)
 {
   return projectReport.ignoredLinks;
 }
 
+
 guint32
 uc_report_get_goodlinks (void)
 {
   return projectReport.goodLinks;
 }
 
+
 guint32
 uc_report_get_timedoutlinks (void)
 {
   return projectReport.timedOutLinks;
 }
 
+
 guint32
 uc_report_get_elapsedtime (void)
 {
   return projectReport.elapsedTime;
 }
 
+
 /*
  * return the total of links
  */
@@ -237,6 +269,7 @@ uc_report_get_alllinks (void)
   return projectReport.allLinks;
 }
 
+
 /*
  * return the number of checked
  * links
@@ -247,6 +280,7 @@ uc_report_get_checkedlinks (void)
   return projectReport.checkedLinks;
 }
 
+
 /*
  * set "ignored links" value
  */
@@ -285,6 +319,7 @@ uc_report_set_goodlinks (const gint32 val)
     projectReport.goodLinks = val;
 }
 
+
 /*
  * set "malformed links" value
  */
@@ -297,6 +332,7 @@ uc_report_set_malformedlinks (const gint32 val)
     projectReport.malformedLinks = val;
 }
 
+
 /*
  * set "bad links" value
  */
@@ -322,12 +358,14 @@ uc_report_set_timedoutlinks (const gint32 val)
     projectReport.timedOutLinks = val;
 }
 
+
 void
 uc_report_set_elapsedtime (const guint32 val)
 {
   projectReport.elapsedTime = val;
 }
 
+
 /*
  * set "single links" value
  */
@@ -350,6 +388,7 @@ uc_check_report_reset (const guint val)
 {
   if (val != 0)
     projectReport.allLinks = 0;
+
   projectReport.checkedLinks = 0;
   projectReport.badLinks = 0;
   projectReport.malformedLinks = 0;
@@ -361,6 +400,7 @@ uc_check_report_reset (const guint val)
   uc_report_display_update ();
 }
 
+
 /*
  * initialize the check report structure
  */
@@ -382,6 +422,7 @@ uc_check_report_force_values (const gint32 all, const gint32 checked,
   uc_report_display_update ();
 }
 
+
 /*
  * update the display check report
  */
@@ -389,19 +430,19 @@ void
 uc_report_display_update (void)
 {
   gchar *tmp = NULL;
-  GtkWidget *widget = NULL;
+
 
   uc_report_set_checkedlinks ();
 
   /* checked links */
-  tmp = (uc_project_get_bookmarks_file ())?
-    g_strdup_printf (_("<i><b>Checked links</b></i>: %d"),
-		     projectReport.checkedLinks) :
-    g_strdup_printf (_("<i><b>Checked links</b></i>: %d / %d"),
-		     projectReport.checkedLinks, projectReport.allLinks);
-
-  widget = WGET ("mw_label_checked_links");
-  gtk_label_set_markup (GTK_LABEL (widget), tmp);
+  if (uc_project_get_check_is_bookmarks ())
+    tmp = g_strdup_printf (_("<i><b>Checked links</b></i>: %d"),
+                           projectReport.checkedLinks);
+  else
+    tmp = g_strdup_printf (_("<i><b>Checked links</b></i>: %d / %d"),
+                           projectReport.checkedLinks,
+                           projectReport.allLinks);
+  gtk_label_set_markup (GTK_LABEL (WGET ("mw_label_checked_links")), tmp);
   g_free (tmp), tmp = NULL;
 
   /* good links */
@@ -446,9 +487,9 @@ report_set_progress_bar (const guint32 report_item, const gchar * glade_label,
   gchar *tmp = NULL;
   GdkColor color;
 
-  percent =
-    (projectReport.checkedLinks) ? (report_item * 100) /
-    projectReport.checkedLinks : 0;
+
+  percent = (projectReport.checkedLinks) ?
+    (report_item * 100) / projectReport.checkedLinks : 0;
 
   widget = WGET (glade_label);
 
@@ -476,7 +517,6 @@ report_set_progress_bar (const guint32 report_item, const gchar * glade_label,
 gboolean
 uc_report_timer_callback (gpointer data)
 {
-  GtkWidget *widget = NULL;
   gchar *tmp = NULL;
   guint seconds = 0;
   guint minutes = 0;
@@ -486,11 +526,9 @@ uc_report_timer_callback (gpointer data)
   minutes = seconds / 60;
   hours = minutes / 60;
 
-  widget = WGET ("mw_label_ellapsed_time");
-  tmp = g_strdup_printf (_
-			 ("<i><b>Elapsed time</b></i>: %02d:%02d:%02d"),
+  tmp = g_strdup_printf (_("<i><b>Elapsed time</b></i>: %02d:%02d:%02d"),
 			 hours % 24, minutes % 60, seconds % 60);
-  gtk_label_set_markup (GTK_LABEL (widget), tmp);
+  gtk_label_set_markup (GTK_LABEL (WGET ("mw_label_ellapsed_time")), tmp);
   g_free (tmp), tmp = NULL;
 
   return TRUE;
