@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2005 
+ * Copyright (C) 2002-2009
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,15 +27,14 @@
 typedef struct _UCClam UCClam;
 struct _UCClam
 {
-  struct cl_node *root;
-  guint no;
-  const gchar *virname;
-  gulong size;
-  struct cl_limits limits;
+  gsize size;
+  unsigned int sigs;
+  const char *virname;
+  struct cl_engine *engine;
 };
 
 UCClam *uc_ucclam_new (void);
-gboolean uc_ucclam_scan (UCClam * clam, const gchar * path);
+gboolean uc_ucclam_scan (UCClam * clam, const gchar * path, gchar ** virname);
 void uc_ucclam_free (UCClam * clam);
 
 #endif
